/*
 * AchievementHandlerGuest
 *
 * This handler is used for fielding achievements requests made while the customer is
 * in guest mode.
 * 
 * TODO: Consider changing the class structure of this handler OR register it with the router to be a proper request handler
 */
AchievementHandlerGuest = (function() {
    var constructor = function(achievementServiceProxyRef, achievementCacheRef) {
	    var requestTypeHandled = 'AchievementRequest';
		var achievementService = achievementServiceProxyRef;
		var achievementCache = achievementCacheRef;
		
		this['getHandledType'] = function() {
		    return requestTypeHandled;
	    };
		
	    this['handleRequest'] = function(request) {
		    /*
			 * Step 1: Ensure request is of the appropriate type
			 * Step 2: Depending on the request's action code, call achievement service proxy
			 * Step 3: Perform any caching or cache lookup as necessary
			 * Step 4: Return data
			 */
			var dfd = $.Deferred();
			
			if (!(request instanceof Request)) {
				console.log('AchievementHandlerGuest: handleRequest: input was not of type Request');
				var result = new Result(constants.NativeCallResultCode.REQUEST_ERROR, {});
		        dfd.resolve(result);
			} else {
			    console.log('AchievementHandlerGuest: handleRequest: message is: ' + JSON.stringify(request.getType()));
			    var requestType = request.getType();
				var requestParams = request.getParams();
				var actionCode = requestParams['action'];
				
				var reply = {};
				switch (actionCode) {
					case 'getAchievement':
					    var isCacheOnly = requestParams['cacheOnly'];
						if (isCacheOnly) {
						    // CACHE LOOKUP
						    achievementCache.getCachedAchievement('SELF', requestParams['achievementId']).always(function(result, resultCode) {
								console.log('AchievementHandlerGuest: getAchievement promise: ' + JSON.stringify(result));
								var replyCode = resultCode; //TODO: Validate service response for true success
								var replyData = result;
								var result = new Result(replyCode, replyData);
								dfd.resolve(result);
							});
						} else {
						    // SERVICE LOOKUP
							achievementService.getAchievement(requestParams['achievementId']).always(function(result, resultCode) {
								console.log('AchievementHandlerGuest: getAchievement promise: ' + JSON.stringify(result));

								if (resultCode != constants.NativeCallResultCode.SUCCESS) {
								    // CACHE FALL-BACK
								    console.log('AchievementHandlerGuest: getAchievement promise: SERVICE RESULT NOT OK');
								    achievementCache.getCachedAchievement('SELF', requestParams['achievementId']).always(function(result, resultCode) {
										console.log('AchievementHandlerGuest: getAchievement promise: ' + JSON.stringify(result));
										var replyCode = resultCode;
										var replyData = result;
										var result = new Result(replyCode, replyData);
										dfd.resolve(result);
							        });
								} else {
								    // SERVICE SUCCESS
									
									// Guest players do not have achievement playerInfo data stored in the cloud.
									// We must either use the cached value, if it exists, or assume the user
									// has no progress on this achievement.
									var serviceResult = result;
                                    achievementCache.getCachedAchievement('SELF', requestParams['achievementId']).always(function(cacheResult, cacheResultCode) {
                                        if (cacheResultCode == constants.NativeCallResultCode.SUCCESS
                                            && cacheResult != null
                                            && cacheResult.achievement != null
                                            && cacheResult.achievement.playerInfo != null) {
                                            serviceResult.achievement.playerInfo = cacheResult.achievement.playerInfo;
                                        } else {
                                            serviceResult.achievement.playerInfo = getBlankPlayerInfo();
                                        }
										
										// Based on the playerInfo, set the appropriate description
										if (serviceResult.achievement.playerInfo['progress'] === 100) {
										    serviceResult.achievement['achievementMeta']['description'] = serviceResult.achievement['achievementMeta']['unlockedDescription'];
										} else {
										    serviceResult.achievement['achievementMeta']['description'] = serviceResult.achievement['achievementMeta']['lockedDescription'];
										}
										
										/* TODO: Use appropriate reply code.  Not every call is a SUCCESS */
									    var replyCode = constants.NativeCallResultCode.SUCCESS;
									    var replyData = serviceResult;
									    var result = new Result(replyCode, replyData);
									
									    achievementCache.cacheAchievement('SELF', requestParams['achievementId'], replyData).always(function(result) {
    								        console.log('UPDATED ACHIEVEMENT CACHE FOR: ' + requestParams['achievementId']);
									    });
									    
									    dfd.resolve(result);
                                    });
								}
							});
						}
						break;
					case 'getAchievements':
					    var isCacheOnly = requestParams['cacheOnly'];
						if (isCacheOnly) {
						    // CACHE LOOKUP
							achievementCache.getCachedAchievements('SELF').always(function(result, resultCode) {
								console.log('AchievementHandlerGuest: getAchievements promise: ' + JSON.stringify(result));
								var replyCode = resultCode;
								var replyData = result;
								var result = new Result(replyCode, replyData);
								dfd.resolve(result);
							});
						} else {
						    // SERVICE LOOKUP
							achievementService.getAchievements().always(function(result, resultCode) {
								console.log('AchievementHandlerGuest: getAchievements promise: ' + JSON.stringify(result));
								if (resultCode != constants.NativeCallResultCode.SUCCESS) {
								    // CACHE FALL-BACK
								    console.log('AchievementHandlerGuest: getAchievements promise: SERVICE RESULT NOT OK');
								    achievementCache.getCachedAchievements('SELF').always(function(result, resultCode) {
										console.log('AchievementHandlerGuest: getAchievements promise: ' + JSON.stringify(result));
										var replyCode = resultCode;
										var replyData = result;
										var result = new Result(replyCode, replyData);
										dfd.resolve(result);
							        });
								} else {
						            // SERVICE SUCCESS
									
									// Guest players do not have achievement playerInfo data stored in the cloud.
									// We must either use the cached value, if it exists, or assume the user
									// has no progress on this achievement.
									console.log('AchievementHandlerGuest: getAchievements: service success');
									console.log('AchievementHandlerGuest: getAchievements: service success: result: ' + JSON.stringify(result));
									var serviceResults = result;
									achievementCache.getCachedAchievements('SELF').always(function(cacheResult, cacheResultCode) {
									    console.log('AchievementHandlerGuest: getCachedAchievements promise: ' + JSON.stringify(serviceResults));
									    console.log('AchievementHandlerGuest: getAchievements: cache reconciliation');
									    var cachedPlayerInfoMap = {};
										if (cacheResultCode == constants.NativeCallResultCode.SUCCESS
										      && cacheResult != null
											  && cacheResult.achievements != null
											  && cacheResult.achievements.achievements != null) {
										    var cachedAchievements = cacheResult.achievements.achievements;
										    $.each(cachedAchievements, function(i, cachedAchievement) {
											    if (cachedAchievement['playerInfo'] != null) {
												    cachedPlayerInfoMap[cachedAchievement['achievementMeta']['id']] = cachedAchievement['playerInfo'];
												}
											});
										}
									
										if (serviceResults != null && serviceResults.achievements != null && serviceResults.achievements.achievements != null) {
									        var achievements = serviceResults.achievements.achievements;
										    $.each(achievements, function(i, achievement) {
                                                if (achievement != null) {
												    var cachedPlayerInfo = cachedPlayerInfoMap[achievement['achievementMeta']['id']]
                                                    if (cachedPlayerInfo == null) {
                                                        achievement.playerInfo = getBlankPlayerInfo();
                                                    } else {
													    achievement.playerInfo = cachedPlayerInfo;
													}
													
													// Based on the playerInfo, set the appropriate description
													if (achievement.playerInfo['progress'] === 100) {
													    achievement['achievementMeta']['description'] = achievement['achievementMeta']['unlockedDescription'];
													} else {
													    achievement['achievementMeta']['description'] = achievement['achievementMeta']['lockedDescription'];
													}
                                                }
                                            });
                                        }
										
										console.log('AchievementHandlerGuest: getAchievements: cache reconciliation complete');
										/* TODO: Use appropriate reply code.  Not every call is a SUCCESS */
									    var replyCode = constants.NativeCallResultCode.SUCCESS;
									    var replyData = serviceResults;
									    var result = new Result(replyCode, replyData);
										
									    achievementCache.cacheAchievements('SELF', replyData).always(function(result) {
    									    console.log('UPDATED ACHIEVEMENTS CACHE');
	    								});
		    							
										dfd.resolve(result);
							        });
								}
							});
						}
						break;
					case 'updateAchievement':
					    // Get cached data
						// If it didn't exist, try to get it from service
						// If we can't get data, just queue and return
						// Otherwise, update, queue and return
                        achievementCache.getCachedAchievement('SELF', requestParams['achievementId']).always(function(result, resultCode) {
						    var achievementGatheringDfd = $.Deferred();
						    if (resultCode != constants.NativeCallResultCode.SUCCESS) {
							    achievementService.getAchievement(requestParams['achievementId']).always(function(serviceResult, serviceResultCode) {
								    if (serviceResultCode == constants.NativeCallResultCode.SUCCESS) {
										serviceResult.achievement.playerInfo = getBlankPlayerInfo();
										// TODO: More error handling
									    achievementCache.cacheAchievement('SELF', requestParams['achievementId'], serviceResult).always(function(result) {
									        achievementGatheringDfd.resolve(serviceResult, serviceResultCode);
									    });
									} else {
										achievementGatheringDfd.resolve(serviceResult, serviceResultCode);
									}
									
								});
							} else {
							    achievementGatheringDfd.resolve(result, resultCode);
							}
						    
						    achievementGatheringDfd.promise().always(function(achievementResult, achievementResultCode) {
							    // We either have an achievement or not, now.  If we do, we'll go with it.  If not,
								// we'll just have to queue the submission
								
							    var cachedProgress = -1;
							    if (achievementResultCode == constants.NativeCallResultCode.SUCCESS && achievementResult
								    && achievementResult['achievement'] && achievementResult['achievement']['playerInfo']
									&& achievementResult['achievement']['playerInfo']['progress']) {
    								cachedProgress = achievementResult['achievement']['playerInfo']['progress'];
							    }
							
							    var progress = Math.min(requestParams['achievementProgress'], 100);
							
							    if (progress > cachedProgress) {
									
									var request = {};
									var jsonEvent = {};
									// TODO: Appropriately abstract type and params for offline event submission
                                    jsonEvent['type'] = 'achievementEvent';
									jsonEvent['achievementId'] = requestParams['achievementId'];
									jsonEvent['progress'] = requestParams['achievementProgress'];
									jsonEvent['eventTime'] = new Date().getTime();

									request['eventJson'] = jsonEvent;
                                    
                                    NativeTransport.callNative({nativeCall: constants.NativeCallTypes.QUEUE_OFFLINE_EVENT, args: request});

									if (!achievementResult || !achievementResult['achievement'] || !achievementResult['achievement']['playerInfo']) {
									    // Queue it up and return
		    					        var replyData = {};
			    					    var replyCode = constants.NativeCallResultCode.SUCCESS;
										if (progress < 100) {
										    replyData['newlyUnlocked'] = false;
										} else {
										    replyData['newlyUnlocked'] = true;
										}
										
				    				    var result = new Result(replyCode, replyData);
					    		        dfd.resolve(result);
                                    } else {
									    achievementResult['achievement']['playerInfo']['progress'] = progress;
										if (progress === 100) {
										    achievementResult['achievement']['playerInfo']['unlocked'] = true;
											achievementResult['achievement']['playerInfo']['dateUnlocked'] = new Date().getTime();
											achievementResult['newlyUnlocked'] = true;
											if (achievementResult['achievement']['achievementMeta']['unlockedDescription']) {
											    achievementResult['achievement']['achievementMeta']['description'] = achievementResult['achievement']['achievementMeta']['unlockedDescription'];
										    }
										} else {
										    achievementResult['newlyUnlocked'] = false;
											if (achievementResult['achievement']['achievementMeta']['lockedDescription']) {
											    achievementResult['achievement']['achievementMeta']['description'] = achievementResult['achievement']['achievementMeta']['lockedDescription'];
											}
										}
										
										/* TODO: Use appropriate reply code.  Not every call is a SUCCESS */
										var replyCode = constants.NativeCallResultCode.SUCCESS;
										
										var replyData = achievementResult;
										
										if (progress < 100) {
										    replyData['newlyUnlocked'] = false;
										} else {
										    replyData['newlyUnlocked'] = true;
										}
										var result = new Result(replyCode, replyData);
											
										achievementCache.updateCachedAchievementProgress('SELF', requestParams['achievementId'], progress).always(function(result, resultCode) {
											console.log('UPDATED ACHIEVEMENTS CACHE');
										});
										
										console.log('AchievementHandlerGuest: updateAchievement updated and returning');
										
										if (progress === 100) {									
											if (replyData && replyData['achievement'] && replyData['achievement']['achievementMeta']) {
												var achievement = result.getResultMap()['achievement'];
												var achievementMetadata = achievement['achievementMeta'];
												var title = achievementMetadata['title'];
												/*var description = achievementMetadata['title'];*/
												var description = '</br>';
												/* TODO: Abstract out toast request creation into a better defined model */
												var toastRequest = {};
												toastRequest['type'] = 'Achievement';
												toastRequest['dedupeTypes'] = ['Achievement', 'Leaderboard'];
												toastRequest['target'] = 'Achievement';
												toastRequest['method'] = 'Unlocked';
												toastRequest['title'] = title;
												toastRequest['description'] = description;
												toastRequest['icon'] = '';
												hostinterface.showToast(JSON.stringify(toastRequest));
											}
										}
										dfd.resolve(result);
									}
	    						} else {
								    console.log('AchievementHandlerRecognized: updateAchievement: not new');
									var result = {};
									result['newlyUnlocked'] = false;
		    					    var replyData = result;
			    					var replyCode = constants.NativeCallResultCode.SUCCESS;

				    				var result = new Result(replyCode, replyData);
					    		    dfd.resolve(result);
						    	}
							});
						});
						break;
					default:
						console.log('AchievementHandlerGuest: handleRequest: actionCode not supported: ' + actionCode);
				        var result = new Result(constants.NativeCallResultCode.REQUEST_ERROR, {});
		                dfd.resolve(result);
				}
		    }
			
			console.log('AchievementHandlerGuest: handleRequest: end of call');
			return dfd.promise();
		};

        var getBlankPlayerInfo = function() {
            return {progress:0, unlocked:false};
        };
    };

    return constructor;
}());
console.log("AchievementHandlerGuest loaded.");