/*
 * AchievementHandlerRecognized
 *
 * This handler is used for fielding achievements requests made while the customer is
 * recognized (not guest).
 * 
 * TODO: Consider changing the class structure of this handler OR register it with the router to be a proper request handler
 */
AchievementHandlerRecognized = (function() {
    var constructor = function(achievementServiceProxyRef, achievementCacheRef) {
	    
	    var requestTypeHandled = 'AchievementRequest';
		var achievementService = achievementServiceProxyRef;
		var achievementCache = achievementCacheRef;
		
		this['getHandledType'] = function() {
		    return requestTypeHandled;
	    };
		
	    this['handleRequest'] = function(request) {
		    /*
			 * Step 1: Ensure request is of the appropriate type
			 * Step 2: Depending on the request's action code, call achievement service proxy
			 * Step 3: Perform any caching or cache lookup as necessary
			 * Step 4: Return data
			 */
			var dfd = $.Deferred();
			
			if (!(request instanceof Request)) {
				console.log('AchievementHandlerRecognized: handleRequest: input was not of type Request');
				var result = new Result(constants.NativeCallResultCode.REQUEST_FAILURE, {});
		        dfd.resolve(result);
			} else {
			    console.log('AchievementHandlerRecognized: handleRequest: message is: ' + JSON.stringify(request.getType()));
			    var requestType = request.getType();
				var requestParams = request.getParams();
				var actionCode = requestParams['action'];
				
				var reply = {};
				switch (actionCode) {
					case 'getAchievement':
					    var isCacheOnly = requestParams['cacheOnly'];
						if (isCacheOnly) {
						    achievementCache.getCachedAchievement('SELF', requestParams['achievementId']).always(function(result, resultCode) {
								console.log('AchievementHandlerRecognized: getAchievement promise: ' + JSON.stringify(result));
								/* TODO: Use appropriate reply code.  Not every call is a SUCCESS */
								var replyCode = resultCode;
								var replyData = result;
								var result = new Result(replyCode, replyData);
								dfd.resolve(result);
							});
						} else {
							achievementService.getAchievement(requestParams['achievementId']).always(function(result, resultCode) {
								console.log('AchievementHandlerRecognized: getAchievement promise: ' + JSON.stringify(result));

								if (resultCode != constants.NativeCallResultCode.SUCCESS) {
								    console.log('AchievementHandlerRecognized: getAchievement promise: SERVICE RESULT NOT OK');
								    achievementCache.getCachedAchievement('SELF', requestParams['achievementId']).always(function(result, resultCode) {
										console.log('AchievementHandlerRecognized: getAchievement promise: ' + JSON.stringify(result));
										/* TODO: Use appropriate reply code.  Not every call is a SUCCESS */
										var replyCode = resultCode;
										var replyData = result;
										var result = new Result(replyCode, replyData);
										dfd.resolve(result);
							        });
								} else {
									/* TODO: Use appropriate reply code.  Not every call is a SUCCESS */
									var replyCode = constants.NativeCallResultCode.SUCCESS;
									var replyData = result;
									var result = new Result(replyCode, replyData);
									
									achievementCache.cacheAchievement('SELF', requestParams['achievementId'], replyData).always(function(result) {
									    /* TODO: Decide if we want this to finish before we resolve) */
									    console.log('UPDATED ACHIEVEMENT CACHE FOR: ' + requestParams['achievementId']);
									});
									dfd.resolve(result);
								}
							});
						}
						break;
					case 'getAchievements':
					    var isCacheOnly = requestParams['cacheOnly'];
						if (isCacheOnly) {
							achievementCache.getCachedAchievements('SELF').always(function(result, resultCode) {
								console.log('AchievementHandlerRecognized: getAchievements promise: ' + JSON.stringify(result));
								/* TODO: Use appropriate reply code.  Not every call is a SUCCESS */
								var replyCode = resultCode;
								var replyData = result;
								var result = new Result(replyCode, replyData);
								dfd.resolve(result);
							});
						} else {
							achievementService.getAchievements().always(function(result, resultCode) {
								console.log('AchievementHandlerRecognized: getAchievements promise: ' + JSON.stringify(result));
								if (resultCode != constants.NativeCallResultCode.SUCCESS) {
								    console.log('AchievementHandlerRecognized: getAchievements promise: SERVICE RESULT NOT OK');
								    achievementCache.getCachedAchievements('SELF').always(function(result, resultCode) {
										console.log('AchievementHandlerRecognized: getAchievements promise: ' + JSON.stringify(result));
										/* TODO: Use appropriate reply code.  Not every call is a SUCCESS */
										var replyCode = resultCode;
										var replyData = result;
										var result = new Result(replyCode, replyData);
										dfd.resolve(result);
							        });
								} else {
									/* TODO: Use appropriate reply code.  Not every call is a SUCCESS */
									var replyCode = constants.NativeCallResultCode.SUCCESS;
									var replyData = result;
									var result = new Result(replyCode, replyData);
									
									achievementCache.cacheAchievements('SELF', replyData).always(function(result) {
									    /* TODO: Decide if we want this to finish before we resolve) */
									    console.log('UPDATED ACHIEVEMENTS CACHE');
									});
									dfd.resolve(result);
								}
							});
						}
						break;
					case 'updateAchievement':
					    achievementCache.getCachedAchievement('SELF', requestParams['achievementId']).always(function(result, resultCode) {
						    /* TODO: Decide how we want to handle cache failure here */
							console.log('AchievementHandlerRecognized: getAchievement promise: ' + JSON.stringify(result));
							
							var cachedProgress = -1;
							if (resultCode == constants.NativeCallResultCode.SUCCESS) {
								cachedProgress = result['achievement']['playerInfo']['progress'];
							}
							
							var progress = Math.min(requestParams['achievementProgress'], 100);
							
							if (progress > cachedProgress) {
							    var cachedAchievement = result;
								achievementService.updateAchievementProgress(requestParams['achievementId'], progress).always(function(result, resultCode) {
								    if (resultCode != constants.NativeCallResultCode.SUCCESS) {
										var request = {};
									    var jsonEvent = {};
									    // TODO: Appropriately abstract type and params for offline event submission
                                        jsonEvent['type'] = 'achievementEvent';
									    jsonEvent['achievementId'] = requestParams['achievementId'];
									    jsonEvent['progress'] = requestParams['achievementProgress'];
									    jsonEvent['eventTime'] = new Date().getTime();

      									request['eventJson'] = jsonEvent;
                                    
                                        NativeTransport.callNative({nativeCall: constants.NativeCallTypes.QUEUE_OFFLINE_EVENT, args: request});
										/* TODO: Consider making this serial so we can message back failure if we failed to queue */
										if (progress < 100) {
										    result['newlyUnlocked'] = false;
										} else {
										    result['newlyUnlocked'] = true;
										}
									}
									console.log('AchievementHandlerRecognized: updateAchievement promise: ' + JSON.stringify(result));
									/* TODO: Use appropriate reply code.  Not every call is a SUCCESS */
									var replyCode = constants.NativeCallResultCode.SUCCESS;
									
									var replyData = result;
									
									var result = new Result(replyCode, replyData);
							
						            achievementCache.updateCachedAchievementProgress('SELF', requestParams['achievementId'], progress).always(function(result, resultCode) {
								        console.log('UPDATED ACHIEVEMENTS CACHE');
							        });

									var toastAchievement = result.getResultMap();
									if (!toastAchievement || !toastAchievement['achievement']) {
									     toastAchievement = cachedAchievement;
								    }
									if (toastAchievement && toastAchievement['achievement'] && toastAchievement['achievement']['achievementMeta'] && toastAchievement['newlyUnlocked']) {
										var achievement = toastAchievement['achievement'];
										var achievementMetadata = achievement['achievementMeta'];
										var title = achievementMetadata['title'];
										/*var description = achievementMetadata['title'];*/
										var description = '</br>';
										/* TODO: Abstract out toast request creation into a better defined model */
										var toastRequest = {};
										toastRequest['type'] = 'Achievement';
										toastRequest['dedupeTypes'] = ['Achievement', 'Leaderboard'];
										toastRequest['target'] = 'Achievement';
										toastRequest['method'] = 'Unlocked';
										toastRequest['title'] = title;
										toastRequest['description'] = description;
										toastRequest['icon'] = '';
										hostinterface.showToast(JSON.stringify(toastRequest));
									}
									
									dfd.resolve(result);
								});
							} else {
								var replyCode = constants.NativeCallResultCode.SUCCESS;
								var result = {};
							    result['newlyUnlocked'] = false;
								
								var replyData = result;
								var result = new Result(replyCode, replyData);
							    dfd.resolve(result);
							}
						});
						break;
					default:
						console.log('AchievementHandlerRecognized: handleRequest: actionCode not supported: ' + actionCode);
				        var result = new Result(constants.NativeCallResultCode.REQUEST_FAILURE, {});
		                dfd.resolve(result);
				}
		    }
			
			console.log('AchievementHandlerRecognized: handleRequest: end of call');
			return dfd.promise();
		};
    };

    return constructor;
}());
console.log("AchievementHandlerRecognized loaded.");