AchievementOfflineCache = (function() {
    var constructor = function() {
		/* public functions */
	    this['getCachedAchievement'] = function(playerId, achievementId) {
		    console.log('AchievementOfflineCache: getCachedAchievement');
			var dfd = $.Deferred();
			var args = {};
			args['primaryKey'] = achievementId + playerId;
			NativeTransport.serviceCall({target: constants.OfflineCacheTypes.ACHIEVEMENTS_CACHE, method: "getCacheItem", args: args}).always(function(responseJson, resultCode) {
			    console.log('AchievementOfflineCache: getCachedAchievement complete: ' + JSON.stringify(responseJson));
				var achievementResponseJson = {"achievement":responseJson};
				dfd.resolve(achievementResponseJson, resultCode);
			});
			
			return dfd.promise();
		};
		this['cacheAchievement'] = function(playerId, achievementId, jsonData) {
		    console.log('AchievementOfflineCache: cacheAchievement');
			var dfd = $.Deferred();
			var args = {};
			var achievementData = jsonData['achievement'];
			/* TODO: There should be a separator between achievement id and playerid on all the caches. */
			var cacheRequest = {'primaryKey': achievementId + playerId,
			                    'secondaryKey': playerId,
								'jsonData': achievementData};
			args['cacheRequest'] = cacheRequest;
			NativeTransport.serviceCall({target: constants.OfflineCacheTypes.ACHIEVEMENTS_CACHE, method: "setCacheItem", args: args}).always(function(responseJson, resultCode) {
			    console.log('AchievementOfflineCache: cacheAchievement complete');
				/* TODO: Validate and decide on return type */
				dfd.resolve(responseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['getCachedAchievements'] = function(playerId) {
		    console.log('AchievementOfflineCache: getCachedAchievements');
			var dfd = $.Deferred();
			var args = {};
			args['secondaryKey'] = playerId;
			NativeTransport.serviceCall({target: constants.OfflineCacheTypes.ACHIEVEMENTS_CACHE, method: "queryCacheItems", args: args}).always(function(responseJson, resultCode) {
			    console.log('AchievementOfflineCache: getCachedAchievements complete: ' + JSON.stringify(responseJson));
				var achievementsReponseJson = {'achievements':{'achievements':responseJson['queryResult']}};
				dfd.resolve(achievementsReponseJson, resultCode);
			});
			
			return dfd.promise();
		};
		this['cacheAchievements'] = function(playerId, jsonData) {
		    console.log('AchievementOfflineCache: cacheAchievements');
			var dfd = $.Deferred();
			var args = {};
			var cacheRequests = [];
			
			var achievements = jsonData['achievements']['achievements']; /* Should be an array */
			$.each(achievements, function(i, achievement) {
			    var achievementId = achievement['achievementMeta']['id'];
				var cacheRequest = {'primaryKey': achievementId + playerId,
			                        'secondaryKey': playerId,
								    'jsonData': achievement};
			    cacheRequests.push(cacheRequest);
			});

			args['cacheRequests'] = cacheRequests;
			
			NativeTransport.serviceCall({target: constants.OfflineCacheTypes.ACHIEVEMENTS_CACHE, method: "setCacheItems", args: args}).always(function(responseJson, resultCode) {
			    console.log('AchievementOfflineCache: cacheAchievements complete');
				/* TODO: Validate and decide on return type */
				dfd.resolve(responseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['updateCachedAchievementProgress'] = function(playerId, achievementId, progress) {
		    var dfd = $.Deferred();
		    console.log('AchievementOfflineCache: updateAchievementProgress');
			var x = this;
			this.getCachedAchievement(playerId, achievementId).always(function(result, resultCode) {
			    if (resultCode == constants.NativeCallResultCode.SUCCESS) {
				    if (result && result['achievement'] && result['achievement']['achievementMeta']) {
					    var isUnlocked = false;
						if (progress === 100) {
						    isUnlocked = true;
						}
						var playerInfo = {'progress':progress, 'unlocked':isUnlocked};
						if (isUnlocked) {
						    playerInfo['dateUnlocked'] = new Date().getTime();
						}
						
						result['achievement']['playerInfo'] = playerInfo;

					    x.cacheAchievement('SELF', achievementId, result).always(function(responseJson, resultCode) {
    						dfd.resolve(responseJson, resultCode);
						});
					} else {
					    var responseJson = {};
				        dfd.resolve(responseJson, constants.NativeCallResultCode.ERROR);
					}
				} else {
				    var responseJson = {};
				    dfd.resolve(responseJson, constants.NativeCallResultCode.ERROR);
				}
			});

			return dfd.promise();
			
		};
		
		this['clear'] = function() {
		    console.log('AchievementOfflineCache: clear');
			var dfd = $.Deferred();
			var args = {};
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.ACHIEVEMENTS_CACHE, method: "clear", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('AchievementOfflineCache: clear complete: ' + JSON.stringify(responseJson));
				dfd.resolve(responseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		/* TODO: Finish implementing loadIcon */
		this['loadIcon'] = function(achievementId, iconSize, isUnlocked) {
		    console.log('AchievementOfflineCache: loadIcon');
		};
    };  
		
    return constructor;
}());
console.log("AchievementOfflineCache loaded.");
