APICallHandler = (function() {
    var constructor = function(ftueHandlerRef, timePlayedHandlerRef, gcSettingsRef) {
        var ftueHandler = ftueHandlerRef;
		var timePlayedHandler = timePlayedHandlerRef;
		var gcSettings = gcSettingsRef;
		var requestTypeHandled = 'APICall';
		
		this['getHandledType'] = function() {
		    return requestTypeHandled;
		}
		
	    this['handleRequest'] = function(request) {
	        // TODO: add try/catch where appropriate (probably multiple files) and resolve deferreds with an error.
		    var dfd = $.Deferred();
			
			if (!(request instanceof Request)) {
				console.log('APICallHandler: handleRequest: input was not of type Request');
				var result = new Result(constants.NativeCallResultCode.ERROR, {});
		        dfd.resolve(result);
			} else {
			    var requestType = request.getType();
				var requestParams = request.getParams();
				
				console.log('APICallHandler: handleRequest: message is: ' + JSON.stringify(requestParams));
				var actionCode = "" + requestParams[constants.ServiceActionCode.ACTION_CODE_KEY];
				var rid = requestParams[constants.BindingKeys.REQUEST_ID_KEY];
				
				var reply = {};
				switch (actionCode) {
                    case constants.ServiceActionCode.AUTHORIZE_SESSION:
                        ftueHandler.onInitialize();
                        ftueHandler.getInitializationPromise().always(function() {
                            var reply = {};
                            reply[constants.BindingKeys.AUTHORIZE_RESULT_BUNDLE_KEY] = constants.AuthorizeKeys.AUTHORIZED;
                            var replyCode = constants.NativeCallResultCode.SUCCESS;
                            var apiResult = new Result(replyCode, reply);
                            dfd.resolve(apiResult);
                        });
                        break;
                    case constants.ServiceActionCode.START_SESSION:
                        timePlayedHandler.startSession();
                        var apiResult = new Result(constants.NativeCallResultCode.SUCCESS, {});
                        dfd.resolve(apiResult);
                        break;
                    case constants.ServiceActionCode.STOP_SESSION:
                        timePlayedHandler.stopSession();
                        var apiResult = new Result(constants.NativeCallResultCode.SUCCESS, {});
                        dfd.resolve(apiResult);
                        break;
                    case constants.ServiceActionCode.SHOW_SIGN_IN_PAGE:
                        ftueHandler.showSignInPage();
                        var apiResult = new Result(constants.NativeCallResultCode.SUCCESS, {});
                        dfd.resolve(apiResult);
                        break;
                    case constants.ServiceActionCode.AUTHENTICATE:
                        ftueHandler.authenticate();
                        var apiResult = new Result(constants.NativeCallResultCode.SUCCESS, {});
                        dfd.resolve(apiResult);
                        break;
                    case constants.ServiceActionCode.UPDATE_COUNTRY_OF_RESIDENCE:
                        ftueHandler.updateCOR(request.getParams()[constants.BindingKeys.COUNTRY_OF_RESIDENCE]);
                        var apiResult = new Result(constants.NativeCallResultCode.SUCCESS, {});
                        dfd.resolve(apiResult);
                        break;
                    case constants.ServiceActionCode.IS_WHISPERSYNC_ENABLED:
                        /* Checks if the user is not a guest and whispersync is enabled in the settings. */
                        /* TODO: This should likely be moved to native now that there's global settings. */
                        gcSettings.getGuestMode().always(function(globalGuestMode) {
                            gcSettings.getWhispersyncEnabledUserSetting().always(function(whispersyncEnabled) {
                                var reply = {};
                                if (globalGuestMode || !whispersyncEnabled) {
                                    reply[constants.BindingKeys.WHISPERSYNC_ENABLED] = false;
                                } else {
                                    reply[constants.BindingKeys.WHISPERSYNC_ENABLED] = true;
                                }
                                var apiResult = new Result(constants.NativeCallResultCode.SUCCESS, reply);
                                dfd.resolve(apiResult);
                            });
                        });
                        break;
					case constants.ServiceActionCode.SHOW_OVERLAY_ACHIEVEMENTS:
						var overlayRequest = {};
						overlayRequest[constants.OverlayBindingKeys.OVERLAY_ACTION_CODE_KEY] = constants.OverlayActionCode.SHOW_ACHIEVEMENTS;
						hostinterface.showOverlay(JSON.stringify(overlayRequest));
						callback(reply); /* TODO: This throws an error since it is undefined.  Is this used by the new UI stuff? */
						break;
				 	case constants.ServiceActionCode.SHOW_OVERLAY_PROFILE:
						var overlayRequest = {};
						overlayRequest[constants.OverlayBindingKeys.OVERLAY_ACTION_CODE_KEY] = constants.OverlayActionCode.SHOW_PROFILE;
						hostinterface.showOverlay(JSON.stringify(overlayRequest));
						break;
					case constants.ServiceActionCode.SHOW_OVERLAY_LEADERBOARDS:
					case constants.ServiceActionCode.SHOW_OVERLAY_RANKS:
						var overlayRequest = {};
						overlayRequest[constants.OverlayBindingKeys.OVERLAY_ACTION_CODE_KEY] = constants.OverlayActionCode.SHOW_LEADERBOARDS;
						/* leaderboard id may be missing. */
						overlayRequest[constants.LeaderboardBindingKeys.LEADERBOARD_ID_KEY] = request.getParams()[constants.LeaderboardBindingKeys.LEADERBOARD_ID_KEY];
						hostinterface.showOverlay(JSON.stringify(overlayRequest));
						callback(reply); /* TODO: This throws an error since it is undefined.  Is this used by the new UI stuff? */
						break;
					/***********************************
				     * OFFLINE EVENT PROCESSING SEGMENT
				     ***********************************/
					case constants.ServiceActionCode.PROCESS_OFFLINE_EVENT:
					    var reply = {};
					    var getProcessOfflineEventRequest = ServiceFactory.getRequestFactory().createProcessOfflineEventRequest(request.getParams()['jsonEvent']);
						ServiceFactory.getRequestRouter().handleRequest(getProcessOfflineEventRequest).always(function(result) {
							dfd.resolve(processProcessOfflineEventResult(result));
						});
						break;
				    /************************
				     * ACHIEVEMENTS SEGMENT
				     ************************/
					case constants.ServiceActionCode.REQUEST_ACHIEVEMENT:
						var getAchievementRequest = ServiceFactory.getRequestFactory().createGetAchievementRequest(request.getParams()['ACHIEVEMENT_ID'], false);
						var promise = ServiceFactory.getRequestRouter().handleRequest(getAchievementRequest);
						promise.done(function(result) {
							dfd.resolve(processGetAchievementResult(result));
						});
						break;
					case constants.ServiceActionCode.REQUEST_ACHIEVEMENTS:
						var getAchievementsRequest = ServiceFactory.getRequestFactory().createGetAchievementsRequest(false);
						var promise = ServiceFactory.getRequestRouter().handleRequest(getAchievementsRequest);
						promise.done(function(result) {
						    dfd.resolve(processGetAchievementsResult(result));
						});
						break;
					case constants.ServiceActionCode.UPDATE_ACHIEVEMENT_PROGRESS:
						var updateAchievementsRequest = ServiceFactory.getRequestFactory().createUpdateAchievementsRequest(request.getParams()['ACHIEVEMENT_ID'], request.getParams()['ACHIEVEMENT_UPDATE_PERCENT']);
						var promise = ServiceFactory.getRequestRouter().handleRequest(updateAchievementsRequest);
						promise.done(function(result) {
							dfd.resolve(processUpdateAchievementProgressResult(result));
						});
						break;
					/************************
					 * LEADERBOARDS SEGMENT
					 ************************/
					case constants.ServiceActionCode.REQUEST_LEADERBOARDS:
						var getLeaderboardsForGameRequest = ServiceFactory.getRequestFactory().createGetLeaderboardsForGameRequest(request.getParams()['GAME_ID'], false);
						var promise = ServiceFactory.getRequestRouter().handleRequest(getLeaderboardsForGameRequest);
						promise.done(function(result) {
						    dfd.resolve(processGetLeaderboardsForGameResponse(result));
						});
						break;
					case constants.ServiceActionCode.REQUEST_PLAYER_SCORE:
					    var getHighScoreRequest = ServiceFactory.getRequestFactory().createGetHighScoreRequest(request.getParams()['GAME_ID'], request.getParams()['LEADERBOARD_ID'], request.getParams()['LEADERBOARD_FILTER'], false);
						var promise = ServiceFactory.getRequestRouter().handleRequest(getHighScoreRequest);
						promise.done(function(result) {
						    dfd.resolve(processGetHighScoreResponse(result));
						});
						break;
					case constants.ServiceActionCode.REQUEST_SCORES:
						var getLeaderboardRanksRequest = ServiceFactory.getRequestFactory().createGetLeaderboardRanksRequest(request.getParams()['GAME_ID'], request.getParams()['LEADERBOARD_ID'], request.getParams()['LEADERBOARD_FILTER'], request.getParams()['LEADERBOARD_START_RANK'], request.getParams()['LEADERBOARD_SCORES_COUNT'], false, false);
						var promise = ServiceFactory.getRequestRouter().handleRequest(getLeaderboardRanksRequest);
						promise.done(function(result) {
						    dfd.resolve(processGetLeaderboardRanksResponse(result, request.getParams()['LEADERBOARD_START_RANK']));
						});
						break;
					case constants.ServiceActionCode.REQUEST_PERCENTILES:
						var getLeaderboardPercentilesRequest = ServiceFactory.getRequestFactory().createGetLeaderboardPercentilesRequest(request.getParams()['GAME_ID'], request.getParams()['LEADERBOARD_ID'], request.getParams()['LEADERBOARD_FILTER'], false, false);
						var promise = ServiceFactory.getRequestRouter().handleRequest(getLeaderboardPercentilesRequest);
						promise.done(function(result) {
						    dfd.resolve(processGetLeaderboardPercentilesResponse(result));
						});
						break;
					case constants.ServiceActionCode.SUBMIT_SCORE:
						var submitScoreRequest = ServiceFactory.getRequestFactory().createSubmitScoreRequest(request.getParams()['LEADERBOARD_ID'], request.getParams()['LEADERBOARD_SUBMIT_SCORE']);
						var promise = ServiceFactory.getRequestRouter().handleRequest(submitScoreRequest);
						promise.done(function(result) {
							dfd.resolve(processSubmitScoreResult(result, request.getParams()['LEADERBOARD_SUBMIT_SCORE']));
						});
						break;
					/************************
					 * PROFILES SEGMENT
					 ************************/
					case constants.ServiceActionCode.REQUEST_LOCAL_PLAYER_PROFILE:
						sendReply = 0;
						var promise = NativeTransport.serviceCall({target: "ProfilesService", method: "getDefaultProfile"});
						promise.done(function(params) {
							var alias = params['alias'];
							
							var reply = {};
							reply[constants.BindingKeys.REQUEST_ID_KEY] = request.getParams()[constants.BindingKeys.REQUEST_ID_KEY];
							reply[constants.ServiceResponseCode.RESPONSE_CODE_KEY] = constants.ServiceResponseCode.SERVICE_REQUEST_SUCCESS;
							reply[constants.ProfilesBindingKeys.ALIAS_KEY] = alias;
							hostinterface.sendReply(JSON.stringify(reply));
						});
						break;
					/************************
					 * PROGRESS SEGMENT
					 ************************/
					case constants.ServiceActionCode.REQUEST_PLAYER_PROGRESS:
						sendReply = 0;
						var promise = NativeTransport.serviceCall({target: "ProgressService", method: "getGameProgressForPlayer"});
						promise.done(function(params) {
							var reply = {};
							reply[constants.BindingKeys.REQUEST_ID_KEY] = request.getParams()[constants.BindingKeys.REQUEST_ID_KEY];
							reply[constants.ServiceResponseCode.RESPONSE_CODE_KEY] = constants.ServiceResponseCode.SERVICE_REQUEST_SUCCESS;
							reply[constants.ProgressBindingKeys.PROGRESS_KEY] = 1;
							reply[constants.ProgressBindingKeys.MAX_PROGRESS_KEY] = 2;
							hostinterface.sendReply(JSON.stringify(reply));
						});
						break;
					case constants.ServiceActionCode.SUBMIT_PLAYER_PROGRESS:
						var args = {};
						args['playerProgress'] = request.getParams()['PROGRESS'];
						sendReply = 0;
						var promise = NativeTransport.serviceCall({target: "ProgressService", method: "updateGameProgressForPlayer", args: args});
						promise.done(function(params) {
							var result = {};
							reply[constants.ServiceResponseCode.RESPONSE_CODE_KEY] = constants.ServiceResponseCode.SERVICE_REQUEST_SUCCESS;
							hostinterface.sendReply(JSON.stringify(reply));
						});
						break;
					default:
						console.log("Message not supported what=" + actionCode);
						console.log('APICallHandler: handleRequest: actionCode not supported');
				        var result = new Result(constants.NativeCallResultCode.ERROR, {});
		                dfd.resolve(result);
				}
			}
			console.log('End of API handler call');
			return dfd.promise();
		};
    };
	
	/*************************************************************************************************
	 * Offline Event Results Processing
	 */
	var processProcessOfflineEventResult = function(result) {
	    var reply = {};
		var replyCode = constants.NativeCallResultCode.ERROR; /* Assume worst case if we don't have other information */

		if (result instanceof Result) {
		    replyCode = result.getType();
		}
		
		var apiResult = new Result(replyCode, reply);
		return apiResult;
	}
		
	/*
	 * End of Offline Event Results Processing
	 *************************************************************************************************/
		
	/*************************************************************************************************
	 * Achievements Results Processing
	 */
	 
	var processGetAchievementsResult = function(result) {
	    var reply = {};
		var replyCode = constants.NativeCallResultCode.ERROR; /* Assume worst case if we don't have other information */

		if (result instanceof Result) {
		    replyCode = result.getType();
			if (result.getType() == constants.NativeCallResultCode.SUCCESS) {
				replyCode = constants.NativeCallResultCode.SUCCESS;
		
				var achievementsList = [];
									
				var responseAchievementsListEntry = result.getResultMap()['achievements'];
				var responseAchievementsList = responseAchievementsListEntry['achievements'];
				$.each(responseAchievementsList, function(i, responseAchievement) {
					var achievement = {};

					var responseAchievementMetadata = responseAchievement['achievementMeta'];
					var playerInfo = responseAchievement['playerInfo'];
										
					achievement['ACHIEVEMENT_ID'] = responseAchievementMetadata['id'];
					achievement['ACHIEVEMENT_TITLE'] = responseAchievementMetadata['title'];
					achievement['ACHIEVEMENT_DESCRIPTION'] = responseAchievementMetadata['description'];
					achievement['ACHIEVEMENT_POINTS'] = responseAchievementMetadata['pointValue'];
					achievement['ACHIEVEMENT_HIDDEN'] = responseAchievementMetadata['hiddenValue'];
					achievement['ACHIEVEMENT_UNLOCKED'] = playerInfo['unlocked'];
					achievement['ACHIEVEMENT_UPDATE_PERCENT'] = playerInfo['progress'];
					achievement['ACHIEVEMENT_POSITION'] = responseAchievementMetadata['position'];
										
					if (achievement['ACHIEVEMENT_UNLOCKED']) {
						achievement['ACHIEVEMENT_DATE_UNLOCKED'] = playerInfo['dateUnlocked'];
					} else {
						achievement['ACHIEVEMENT_DATE_UNLOCKED'] = 0;
					}
					achievementsList.push(achievement);
				});
									
				reply[constants.AchievementsBindingKeys.ACHIEVEMENT_LIST_KEY] = achievementsList;
			}
		}
		
		var apiResult = new Result(replyCode, reply);
		return apiResult;
	}
		
	var processGetAchievementResult = function(result) {
	    var reply = {};
		var replyCode = constants.NativeCallResultCode.ERROR; /* Assume worst case if we don't have other information */
							
		if (result instanceof Result) {
		    replyCode = result.getType();
			if (result.getType() == constants.NativeCallResultCode.SUCCESS) {
				replyCode = constants.NativeCallResultCode.SUCCESS;
									
				var achievementJson = result.getResultMap()['achievement'];
				var achievementMetadataJson = achievementJson['achievementMeta'];
				var playerInfoJson = achievementJson['playerInfo'];
				
				reply[constants.AchievementsBindingKeys.ACHIEVEMENT_ID_KEY] = achievementMetadataJson['id'];
    			reply[constants.AchievementsBindingKeys.ACHIEVEMENT_TITLE_KEY] = achievementMetadataJson['title'];
				reply[constants.AchievementsBindingKeys.ACHIEVEMENT_DESCRIPTION_KEY] = achievementMetadataJson['description'];
				reply[constants.AchievementsBindingKeys.ACHIEVEMENT_POINTS_KEY] = achievementMetadataJson['pointValue'];
				reply[constants.AchievementsBindingKeys.ACHIEVEMENT_HIDDEN_KEY] = achievementMetadataJson['hiddenValue'];
				reply[constants.AchievementsBindingKeys.ACHIEVEMENT_POSITION_KEY] = achievementMetadataJson['position'];
				if (playerInfoJson != null) {
    				reply[constants.AchievementsBindingKeys.ACHIEVEMENT_UNLOCKED_KEY] = playerInfoJson['unlocked'];
    				reply[constants.AchievementsBindingKeys.ACHIEVEMENT_UPDATE_PERCENT_KEY] = playerInfoJson['progress'];
    
    				if (reply[constants.AchievementsBindingKeys.ACHIEVEMENT_UNLOCKED_KEY]) {
    					reply[constants.AchievementsBindingKeys.ACHIEVEMENT_DATE_UNLOCKED_KEY] = playerInfoJson['dateUnlocked'];
    				} else {
    					reply[constants.AchievementsBindingKeys.ACHIEVEMENT_DATE_UNLOCKED_KEY] = 0;
    				}
				}
			}
		}
							
		var apiResult = new Result(replyCode, reply);
		return apiResult;
	}
	
	var processUpdateAchievementProgressResult = function(result) {
	    var reply = {};
		var replyCode = constants.NativeCallResultCode.ERROR; /* Assume worst case if we don't have other information */
							
		if (result instanceof Result) {
		    replyCode = result.getType();
			if (result.getType() == constants.NativeCallResultCode.SUCCESS) {
				replyCode = constants.NativeCallResultCode.SUCCESS;
									
				var isNewlyUnlocked = result.getResultMap()['newlyUnlocked'];
				reply[constants.AchievementsBindingKeys.ACHIEVEMENT_NEWLY_UNLOCKED_KEY] = isNewlyUnlocked;
			}
		}
							
		var apiResult = new Result(replyCode, reply);
		return apiResult;
	}
	
	/*
	 * End of Achievements Results Processing
	 *************************************************************************************************/
	 
	/*************************************************************************************************
	 * Leaderboards Results Processing
	 */
	
	var processGetLeaderboardsForGameResponse = function(result) {
	    var reply = {};
		var replyCode = constants.NativeCallResultCode.ERROR; /* Assume worst case if we don't have other information */

		if (result instanceof Result) {
		    replyCode = result.getType();
			if (result.getType() == constants.NativeCallResultCode.SUCCESS) {
				replyCode = constants.NativeCallResultCode.SUCCESS;
		
				var leaderboardsList = [];
				var responseLeaderboardsListEntry = result.getResultMap()['LeaderboardList'];
				$.each(responseLeaderboardsListEntry, function(i, responseLeaderboard) {
					var leaderboard = {};

					var responseLeaderboardInfo = responseLeaderboard['LeaderboardInfo'];
								
				    leaderboard['LEADERBOARD_ID'] = responseLeaderboardInfo['LeaderboardId'];
					leaderboard['LEADERBOARD_NAME'] = responseLeaderboardInfo['Name'];
					leaderboard['LEADERBOARD_DISPLAY_TEXT'] = responseLeaderboardInfo['DisplayText'];
					leaderboard['LEADERBOARD_DATA_FORMAT'] = responseLeaderboardInfo['DataFormat'];
								
					leaderboardsList.push(leaderboard);
				});
									
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_LIST_KEY] = leaderboardsList;
			}
		}
		
		var apiResult = new Result(replyCode, reply);
		return apiResult;
	}
	
	var processGetHighScoreResponse = function(result) {
	    var reply = {};
		var replyCode = constants.NativeCallResultCode.ERROR; /* Assume worst case if we don't have other information */
		
		if (result instanceof Result) {
		    replyCode = result.getType();
			if (result.getType() == constants.NativeCallResultCode.SUCCESS) {
				replyCode = constants.NativeCallResultCode.SUCCESS;
		
				var score = result.getResultMap()['PlayerScore'];
				var rank = result.getResultMap()['PlayerRank'];
									
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_SUBMIT_SCORE_KEY] = score;
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_PLAYER_RANK_KEY] = rank;
			}
		}
		
		var apiResult = new Result(replyCode, reply);
		return apiResult;
	}
	
	var processGetLeaderboardRanksResponse = function(result, startRank) {
	    var reply = {};
		var replyCode = constants.NativeCallResultCode.ERROR; /* Assume worst case if we don't have other information */
		
		if (result instanceof Result) {
		    replyCode = result.getType();
			if (result.getType() == constants.NativeCallResultCode.SUCCESS) {
			    replyCode = constants.NativeCallResultCode.SUCCESS;
			
                var leaderboardInfo = result.getResultMap()['LeaderboardInfo'];
				var rankList = result.getResultMap()['RankList'];
							
				var numScores = rankList.length;
				var leaderboardDisplayText = leaderboardInfo['DisplayText'];
				var leaderboardScoreFormat = leaderboardInfo['DataFormat'];
				var leaderboardName = leaderboardInfo['Name'];
				var leaderboardId = leaderboardInfo['LeaderboardId'];
							
				var rankModelList = [];
				var rankCount = startRank;
							
				var scoreList = [];
							
				$.each(rankList, function(i, rankEntry) {
					var rank = rankCount;
					var scoreValue = rankEntry['Score'];
					var alias = rankEntry['Username'];
								
					var scoreEntry = {};
					scoreEntry['scoreValue'] = scoreValue;
					scoreEntry['rank'] = rank;
					scoreEntry['leaderboard'] = leaderboardId;
					scoreEntry['alias'] = alias;
					scoreList.push(scoreEntry);
								
					rankCount++;
				});
							
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_DISPLAY_TEXT_KEY] = leaderboardDisplayText;
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_DATA_FORMAT_KEY] = leaderboardScoreFormat;
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_NAME_KEY] = leaderboardName;
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_ID_KEY] = leaderboardId;
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_SCORES_ARRAY_KEY] = scoreList;
			}
		}
		
		var apiResult = new Result(replyCode, reply);
		return apiResult;
	}
	
	var processGetLeaderboardPercentilesResponse = function(result) {
	    var reply = {};
		var replyCode = constants.NativeCallResultCode.ERROR; /* Assume worst case if we don't have other information */
		
		if (result instanceof Result) {
		    replyCode = result.getType();
			if (result.getType() == constants.NativeCallResultCode.SUCCESS) {
				replyCode = constants.NativeCallResultCode.SUCCESS;
	
				var leaderboardInfo = result.getResultMap()['LeaderboardInfo'];
				var userIndex = result.getResultMap()['UserIndex'];
				var requestedPercentileList = result.getResultMap()['PercentileList'];

				var percentileList = [];
				$.each(requestedPercentileList, function(i, requestedPercentileEntry) {
					var percentile = requestedPercentileEntry['Percentile'];
					var alias = requestedPercentileEntry['PlayerName'];
					var score = requestedPercentileEntry['PlayerScore'];
								
					var percentileEntry = {};
					percentileEntry['Percentile'] = percentile;
					percentileEntry['PlayerName'] = alias;
					percentileEntry['PlayerScore'] = score;
					percentileList.push(percentileEntry);
				});

				reply[constants.LeaderboardBindingKeys.LEADERBOARD_ID_KEY] = leaderboardInfo['LeaderboardId'];
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_NAME_KEY] = leaderboardInfo['Name'];
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_DISPLAY_TEXT_KEY] = leaderboardInfo['DisplayText'];
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_DATA_FORMAT_KEY] = leaderboardInfo['DataFormat'];
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_PERCENTILES_ARRAY_KEY] = percentileList;
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_USER_INDEX_KEY] = userIndex;
			}
		}
		
		var apiResult = new Result(replyCode, reply);
		return apiResult;
	}
	
	var processSubmitScoreResult = function(result, playerScore) {
	    var reply = {};
		var replyCode = constants.NativeCallResultCode.ERROR; /* Assume worst case if we don't have other information */
		
		if (result instanceof Result) {
		    replyCode = result.getType();
			if (result.getType() == constants.NativeCallResultCode.SUCCESS) {
				replyCode = constants.NativeCallResultCode.SUCCESS;

				var submitScoreResultsList = result.getResultMap()['SubmitScoreResultsList'];
								
				reply[constants.LeaderboardBindingKeys.LEADERBOARD_SUBMIT_RESULT_KEY] = submitScoreResultsList;
			}
		}
		
		var apiResult = new Result(replyCode, reply);
		return apiResult;
	}
	
	/*
	 * End of Achievements Results Processing
	 *************************************************************************************************/
		
    return constructor;
}());
console.log("APICallHandler loaded.");
