LeaderboardHandlerRecognized = (function() {
    var constructor = function(leaderboardServiceProxyRef, leaderboardCacheRef) {
	    var requestTypeHandled = 'LeaderboardRequest';
		var leaderboardService = leaderboardServiceProxyRef;
		var leaderboardCache = leaderboardCacheRef;
		
		this['getHandledType'] = function() {
		    return requestTypeHandled;
	    }
		
	    this['handleRequest'] = function(request) {
		    /*
			 * Step 1: Ensure request is of the appropriate type
			 * Step 2: Depending on the request's action code, call achievement service proxy
			 * Step 3: Perform any caching or cache lookup as necessary
			 * Step 4: Return data
			 */
			var dfd = $.Deferred();
			
			if (!(request instanceof Request)) {
				console.log('LeaderboardHandlerRecognized: handleRequest: input was not of type Request');
				var result = new Result(constants.NativeCallResultCode.REQUEST_ERROR, {});
		        dfd.resolve(result);
			} else {
			    console.log('LeaderboardHandlerRecognized: handleRequest: message is: ' + JSON.stringify(request.getType()));
			    var requestType = request.getType();
				var requestParams = request.getParams();
				var actionCode = requestParams['action'];
				
				var reply = {};
				switch (actionCode) {
					case 'getLeaderboardsForGame':
					    var isCacheOnly = requestParams['cacheOnly'];
						if (isCacheOnly) {
						    promise = leaderboardCache.getLeaderboards('SELF', requestParams['gameId']);
							promise.done(function(result, resultCode) {
								console.log('LeaderboardHandlerRecognized: cached getLeaderboards promise: ' + JSON.stringify(result));
								var replyCode = constants.NativeCallResultCode.SUCCESS;
								var replyData = result;
								var result = new Result(replyCode, replyData);
								dfd.resolve(result);
							});
						} else {
							promise = leaderboardService.getLeaderboardsForGame(requestParams['gameId']);
							promise.done(function(result, resultCode) {
								console.log('LeaderboardHandlerRecognized: getLeaderboardsForGame promise: ' + JSON.stringify(result));
								if (resultCode != constants.NativeCallResultCode.SUCCESS) {
								    console.log('LeaderboardHandlerRecognized: getLeaderboardsForGame promise: SERVICE RESULT NOT OK');
								    promise = leaderboardCache.getLeaderboards('SELF', requestParams['gameId']);
							        promise.done(function(result, resultCode) {
										console.log('LeaderboardHandlerRecognized: cached getLeaderboards promise: ' + JSON.stringify(result));
										var replyCode = constants.NativeCallResultCode.SUCCESS;
										var replyData = result;
										var result = new Result(replyCode, replyData);
										dfd.resolve(result);
							        });
								} else {
									var replyCode = constants.NativeCallResultCode.SUCCESS;
									var replyData = result;
									var result = new Result(replyCode, replyData);
									
									var cachePromise = leaderboardCache.deleteLeaderboards('SELF', requestParams['gameId']);
									
									/* TODO: Decide if we want to make this resolve post-updates */
									cachePromise.done(function(result) {
										var cacheLeaderboardsPromise = leaderboardCache.cacheLeaderboards('SELF', requestParams['gameId'], replyData);
									    cacheLeaderboardsPromise.done(function(result) {
											console.log('UPDATED LEADERBOARDS CACHE cacheLeaderboards');
											var leaderboardList = replyData['LeaderboardList'];
										    var leaderboardInfoList = [];
										    $.each(leaderboardList, function(i, leaderboard) {
  										        var leaderboardInfo = leaderboard['LeaderboardInfo'];
											    leaderboardInfoList.push(leaderboardInfo);
										    });
										
										    /* TODO: Ensure we have a solid interface going back re: errors */
										    var leaderboardInfoCachePromise = leaderboardCache.cacheLeaderboardInfoItems(requestParams['gameId'], leaderboardInfoList);
										});
									});
									
									dfd.resolve(result);
								}
							});
						}
						break;
					case 'getHighScore':
					    var isCacheOnly = requestParams['cacheOnly'];
						if (isCacheOnly) {
							promise = leaderboardCache.getLocalPlayerScore('SELF', requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope']);
							promise.done(function(result, resultCode) {
								console.log('LeaderboardHandlerRecognized: cached getLocalPlayerScore promise: ' + JSON.stringify(result));
								var replyCode = constants.NativeCallResultCode.SUCCESS;
								var replyData = result;
								var result = new Result(replyCode, replyData);
								dfd.resolve(result);
							});
						} else {
							promise = leaderboardService.getHighScore(requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope']);
							promise.done(function(result, resultCode) {
								console.log('LeaderboardHandlerRecognized: getHighScore promise: ' + JSON.stringify(result));

								if (resultCode != constants.NativeCallResultCode.SUCCESS) {
								    console.log('LeaderboardHandlerRecognized: getHighScore promise: SERVICE RESULT NOT OK');
								    promise = leaderboardCache.getLocalPlayerScore('SELF', requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope']);
							        promise.done(function(result, resultCode) {
										console.log('LeaderboardHandlerRecognized: cached getLocalPlayerScore promise: ' + JSON.stringify(result));
										var replyCode = constants.NativeCallResultCode.SUCCESS;
										var replyData = result;
										var result = new Result(replyCode, replyData);
										dfd.resolve(result);
							        });
								} else {
									var replyCode = constants.NativeCallResultCode.SUCCESS;
									var replyData = result;
									var result = new Result(replyCode, replyData);
									
									var cachePromise = leaderboardCache.cacheLocalPlayerScore('SELF', requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope'], replyData);
									cachePromise.done(function(result) {
									    console.log('UPDATED LEADERBOARDS CACHE');
									});
									dfd.resolve(result);
								}
							});
						}
						break;
					case 'getHighestScoreSubmission':
					    leaderboardCache.getHighestScoreSubmissions('SELF').always(function(result, resultCode) {
    						console.log('LeaderboardHandlerGuest: cached getHighestScoreSubmissions promise: ' + JSON.stringify(result));
	    					var replyCode = constants.NativeCallResultCode.SUCCESS;
							var replyData = result;
							var result = new Result(replyCode, replyData);
							dfd.resolve(result);
				        });
						break;
					case 'getLeaderboardRanks':
					    var isCacheOnly = requestParams['cacheOnly'];
						if (isCacheOnly) {
							promise = leaderboardCache.getScores('SELF', requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope'], requestParams['startRank'], requestParams['numRanks'], false);
							promise.done(function(result, resultCode) {
								console.log('LeaderboardHandlerRecognized: cached getScores promise: ' + JSON.stringify(result));
								var replyCode = constants.NativeCallResultCode.SUCCESS;
								var replyData = result;
								var result = new Result(replyCode, replyData);
								dfd.resolve(result);
							});
						} else {
							promise = leaderboardService.getLeaderboardRanks(requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope'], requestParams['startRank'], requestParams['numRanks']);
							promise.done(function(result, resultCode) {
								console.log('LeaderboardHandlerRecognized: getLeaderboardRanks promise: ' + JSON.stringify(result));

								if (resultCode != constants.NativeCallResultCode.SUCCESS) {
								    console.log('LeaderboardHandlerRecognized: getLeaderboardRanks promise: SERVICE RESULT NOT OK');
								    promise = leaderboardCache.getScores('SELF', requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope'], requestParams['startRank'], requestParams['numRanks'], false);
							        promise.done(function(result, resultCode) {
										console.log('LeaderboardHandlerRecognized: cached getScores promise: ' + JSON.stringify(result));
										hostinterface.logMessage('LeaderboardHandlerRecognized: getScores result came back: ' + JSON.stringify(result));
										var replyCode = constants.NativeCallResultCode.SUCCESS;
										var replyData = result;
										var result = new Result(replyCode, replyData);
										dfd.resolve(result);
							        });
								} else {
									var replyCode = constants.NativeCallResultCode.SUCCESS;
									var replyData = result;
									var result = new Result(replyCode, replyData);
									var cachePromise = leaderboardCache.cacheScoresForGame('SELF', requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope'], requestParams['startRank'], requestParams['numRanks'], false, replyData);
									cachePromise.done(function(result) {
									    console.log('UPDATED LEADERBOARDS CACHE');
									});
									dfd.resolve(result);
								}
							});
						}
						break;
					case 'getLeaderboardPercentiles':
					    var isCacheOnly = requestParams['cacheOnly'];
						if (isCacheOnly) {
							promise = leaderboardCache.getPercentiles('SELF', requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope']);
							promise.done(function(result, resultCode) {
								console.log('LeaderboardHandlerRecognized: cached getPercentiles promise: ' + JSON.stringify(result));
								var replyCode = constants.NativeCallResultCode.SUCCESS;
								var replyData = result;
								var result = new Result(replyCode, replyData);
								dfd.resolve(result);
							});
						} else {
							promise = leaderboardService.getLeaderboardPercentiles(requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope']);
							promise.done(function(result, resultCode) {
								console.log('LeaderboardHandlerRecognized: getLeaderboardPercentiles promise: ' + JSON.stringify(result));
								if (resultCode != constants.NativeCallResultCode.SUCCESS) {
								    console.log('LeaderboardHandlerRecognized: getLeaderboardPercentiles promise: SERVICE RESULT NOT OK');
								    promise = leaderboardCache.getPercentiles('SELF', requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope']);
							        promise.done(function(result, resultCode) {
										console.log('LeaderboardHandlerRecognized: cached getPercentiles promise: ' + JSON.stringify(result));
										var replyCode = constants.NativeCallResultCode.SUCCESS;
										var replyData = result;
										var result = new Result(replyCode, replyData);
										dfd.resolve(result);
							        });
								} else {
									var replyCode = constants.NativeCallResultCode.SUCCESS;
									var replyData = result;
									var result = new Result(replyCode, replyData);
									
									var cachePromise = leaderboardCache.cachePercentiles('SELF', requestParams['gameId'], requestParams['leaderboardId'], requestParams['scope'], replyData);
									cachePromise.done(function(result) {
									    console.log('UPDATED LEADERBOARDS CACHE');
									});
									dfd.resolve(result);
								}
							}); 
						}
						break;
					case 'submitScore':
					    var isLocalHighScore = false;
						var cachedLeaderboardInfo;
						var dfd1 = $.Deferred();
						leaderboardCache.getHighestScoreSubmission('SELF', requestParams['gameId'], requestParams['leaderboardId']).always(function(getHighestScoreSubmissionResult, getHighestScoreSubmissionResultCode) {
							if (getHighestScoreSubmissionResultCode != constants.NativeCallResultCode.SUCCESS) {
							    isLocalHighScore = true;
							}
						    leaderboardCache.getLeaderboardInfo(requestParams['gameId'], requestParams['leaderboardId']).always(function(getLeaderboardInfoResult, getLeaderboardInfoResultCode) {
							    cachedLeaderboardInfo = getLeaderboardInfoResult;
								if (getHighestScoreSubmissionResultCode == constants.NativeCallResultCode.SUCCESS && getLeaderboardInfoResultCode == constants.NativeCallResultCode.SUCCESS && getLeaderboardInfoResult && getLeaderboardInfoResult['LeaderboardInfo']) {
									var sortOrder = getLeaderboardInfoResult['LeaderboardInfo']['SortOrder'];
									if (sortOrder == 'DESCENDING') {
										if (requestParams['PlayerScore'] > getHighestScoreSubmissionResult) {
										    isLocalHighScore = true;
										}
									} else {
										if (requestParams['PlayerScore'] < getHighestScoreSubmissionResult) {
										    isLocalHighScore = true;
										}
									}
								}
								dfd1.resolve();
							});
							
							$.when(dfd1).always(function() {
								leaderboardService.submitScore(requestParams['gameId'], requestParams['leaderboardId'], requestParams['PlayerScore']).always(function(result, resultCode) {
									console.log('LeaderboardHandlerRecognized: submitScore promise: ' + JSON.stringify(result));
									if (resultCode != constants.NativeCallResultCode.SUCCESS) {
										var request = {};
										var jsonEvent = {};
										// TODO: Appropriately abstract type and params for offline event submission
										jsonEvent['type'] = 'leaderboardEvent';
										jsonEvent['leaderboardId'] = requestParams['leaderboardId'];
										jsonEvent['PlayerScore'] = requestParams['PlayerScore'];
										jsonEvent['gameId'] = requestParams['gameId'];
										jsonEvent['eventTime'] = new Date().getTime();

										request['eventJson'] = jsonEvent;
											
										NativeTransport.callNative({nativeCall: constants.NativeCallTypes.QUEUE_OFFLINE_EVENT, args: request});

										var leaderboardInfo = cachedLeaderboardInfo;
										if (isLocalHighScore && leaderboardInfo && leaderboardInfo['Name']) {
										console.log('LeaderboardHandlerRecognized: displaying toast even though we just queued');
											var leaderboardName = leaderboardInfo['Name'];
											/* TODO: Abstract out toast request creation into a better defined model */
											var toastRequest = {};
											toastRequest['type'] = 'Leaderboard';
											toastRequest['dedupeTypes'] = ['Achievement', 'Leaderboard'];
											toastRequest['target'] = 'Leaderboard';
											toastRequest['method'] = 'GlobalHigh';
											toastRequest['title'] = leaderboardName;
											toastRequest['description'] = requestParams['PlayerScore'];
											//toastRequest['icon'] = leaderboardInfo['IconUrl'];
											toastRequest['icon'] = '';
											hostinterface.showToast(JSON.stringify(toastRequest));
										}
										
										leaderboardCache.updateCachedScoreSubmission('SELF', requestParams['gameId'], requestParams['leaderboardId'], requestParams['PlayerScore']).always(function(result, resultCode) {
											// TODO: Decide if we'd like to do anything with the result
											console.log('LeaderboardHandlerRecognized: submitScore updateCachedScoreSubmission completed');
										});
										dfd.resolve(result);
									} else {
										var replyCode = constants.NativeCallResultCode.SUCCESS;
										var replyData = result;
										var result = new Result(replyCode, replyData);
										
										leaderboardCache.updateCachedScoreSubmission('SELF', requestParams['gameId'], requestParams['leaderboardId'], requestParams['PlayerScore']).always(function(result, resultCode) {
											// TODO: Decide if we'd like to do anything with the result
											console.log('LeaderboardHandlerRecognized: submitScore updateCachedScoreSubmission completed');
										});
										
										var submitScoreResultsList = result.getResultMap()['SubmitScoreResultsList'];
									
										var leaderboardInfo = result.getResultMap()['LeaderboardInfo'];
										var leaderboardName = leaderboardInfo['Name'];
										$.each(submitScoreResultsList, function(i, submitScoreResult) {
											var isImproved = submitScoreResult['IsImproved'];
											var scope = submitScoreResult['Scope'];
											if (isImproved) {
												if (scope=='GLOBAL_ALL_TIME') {
													/* TODO: Abstract out toast request creation into a better defined model */
													var toastRequest = {};
													toastRequest['type'] = 'Leaderboard';
													toastRequest['dedupeTypes'] = ['Achievement', 'Leaderboard'];
													toastRequest['target'] = 'Leaderboard';
													toastRequest['method'] = 'GlobalHigh';
													toastRequest['title'] = leaderboardName;
													toastRequest['description'] = requestParams['PlayerScore'];
													toastRequest['icon'] = '';
													hostinterface.showToast(JSON.stringify(toastRequest));
												}
											}
										});
										dfd.resolve(result);
									}
								});
							});
						});
						break;
					default:
						console.log('LeaderboardHandlerRecognized: handleRequest: actionCode not supported: ' + actionCode);
				        var result = new Result(constants.NativeCallResultCode.REQUEST_ERROR, {});
		                dfd.resolve(result);
				}
		    }
			
			console.log('LeaderboardHandlerRecognized: handleRequest: end of call');
			return dfd.promise();
		};
    };

    return constructor;
}());
console.log("LeaderboardHandlerRecognized loaded.");