LeaderboardOfflineCache = (function() {
    var constructor = function() {
		/* public functions */
		this['getScores'] = function(playerId, gameId, leaderboardId, leaderboardFilter, startRank, count, friendsEnabled) {
		    console.log('LeaderboardOfflineCache: getScores');
			var dfd = $.Deferred();
			var args = {};
			args['primaryKey'] = "scores-" + playerId + "-" + leaderboardId + "-" + leaderboardFilter + "-" + startRank + "-" + friendsEnabled;
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "getCacheItem", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: getScores complete: ' + JSON.stringify(responseJson));
				/* TODO: Follow these steps */
				/* Step 1: Check resultCodes for validation */
				/* Step 2: Attempt to construct the response objects with the data available */
				/* Step 3: Depending on the outcome of steps 1 and 2, declare this a success or a failure and tag the result with that data */
				/* Step 4: Call the callback provided with our response */
				var scoresResponseJson = responseJson;
				dfd.resolve(scoresResponseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['cacheScoresForGame'] = function(playerId, gameId, leaderboardId, leaderboardFilter, startRank, count, friendsEnabled, jsonData) {
		    console.log('LeaderboardOfflineCache: cacheScoresForGame');
			var dfd = $.Deferred();
			var args = {};
			var cacheRequest = {'primaryKey': "scores-" + playerId + "-" + leaderboardId + "-" + leaderboardFilter + "-" + startRank + "-" + friendsEnabled,
			                    'secondaryKey': "scores-" + playerId + "-" + leaderboardId + "-" + leaderboardFilter + "-" + startRank + "-" + friendsEnabled,
								'jsonData': jsonData};
			args['cacheRequest'] = cacheRequest;
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "setCacheItem", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: cacheScoresForGame complete');
				/* TODO: Validate and decide on return type */
				
				dfd.resolve(responseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['getPercentiles'] = function(playerId, gameId, leaderboardId, leaderboardFilter) {
		    console.log('LeaderboardOfflineCache: getPercentiles');
			var dfd = $.Deferred();
			var args = {};
			args['primaryKey'] = "percentiles-" + playerId + "-" + leaderboardId + "-" + leaderboardFilter;
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "getCacheItem", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: getPercentiles complete: ' + JSON.stringify(responseJson));
				var scoresResponseJson = responseJson;
				dfd.resolve(scoresResponseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['cachePercentiles'] = function(playerId, gameId, leaderboardId, leaderboardFilter, jsonData) {
		    console.log('LeaderboardOfflineCache: cachePercentiles');
			var dfd = $.Deferred();
			var args = {};
			var cacheRequest = {'primaryKey': "percentiles-" + playerId + "-" + leaderboardId + "-" + leaderboardFilter,
			                    'secondaryKey': "percentiles-" + playerId + "-" + leaderboardId,
								'jsonData': jsonData};
			args['cacheRequest'] = cacheRequest;
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "setCacheItem", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: cachePercentiles complete');
				dfd.resolve(responseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['getLocalPlayerScore'] = function(playerId, gameId, leaderboardId, leaderboardFilter) {
		    console.log('LeaderboardOfflineCache: getLocalPlayerScore');
			var dfd = $.Deferred();
			var args = {};
			args['primaryKey'] = "localScore-" + leaderboardId + "-" + leaderboardFilter + "-" + playerId;
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "getCacheItem", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: getLocalPlayerScore complete: ' + JSON.stringify(responseJson));
				var scoresResponseJson = responseJson;
				dfd.resolve(scoresResponseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['cacheLocalPlayerScore'] = function(playerId, gameId, leaderboardId, leaderboardFilter, jsonData) {
		    console.log('LeaderboardOfflineCache: cacheLocalPlayerScore');
			var dfd = $.Deferred();
			var args = {};
			var cacheRequest = {'primaryKey': "localScore-" + leaderboardId + "-" + leaderboardFilter + "-" + playerId,
			                    'secondaryKey': "localScore-" + leaderboardId + "-" + leaderboardFilter + "-" + playerId,
								'jsonData': jsonData};
			args['cacheRequest'] = cacheRequest;
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "setCacheItem", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: cacheLocalPlayerScore complete');
				dfd.resolve(responseJson, resultCode);
			});
			
			return dfd.promise();
		};
	
		this['getLeaderboards'] = function(playerId, gameId) {
		    console.log('LeaderboardOfflineCache: getLeaderboards');
			var dfd = $.Deferred();
			var args = {};
			args['secondaryKey'] = "leaderModel-" + playerId;
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "queryCacheItems", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: getLeaderboards complete: ' + JSON.stringify(responseJson));
				var leaderboardsReponseJson = {'LeaderboardList':responseJson['queryResult']};
				dfd.resolve(leaderboardsReponseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['cacheLeaderboards'] = function(playerId, gameId, jsonData) {
		    console.log('LeaderboardOfflineCache: cacheLeaderboards');
			var dfd = $.Deferred();
			var args = {};
			var cacheRequests = [];
			
			var leaderboards = jsonData['LeaderboardList']; /* Should be an array */
			$.each(leaderboards, function(i, leaderboard) {
			    var leaderboardId = leaderboard['LeaderboardInfo']['LeaderboardId'];
				var cacheRequest = {'primaryKey': "leaderModel-" + leaderboardId + "-" + playerId,
			                        'secondaryKey': "leaderModel-" + playerId,
								    'jsonData': leaderboard};
			    cacheRequests.push(cacheRequest);
			});

			args['cacheRequests'] = cacheRequests;
			
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "setCacheItems", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: cacheLeaderboards complete');
				/* TODO: Validate and decide on return type */
				dfd.resolve(responseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['getLeaderboardInfo'] = function(gameId, leaderboardId) {
		    console.log('LeaderboardOfflineCache: getLeaderboardInfo');
			var dfd = $.Deferred();
			var args = {};
			args['primaryKey'] = "leaderMeta-" + leaderboardId;
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "getCacheItem", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: getLeaderboardInfo complete: ' + JSON.stringify(responseJson));
				var scoresResponseJson = responseJson;
				dfd.resolve(scoresResponseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['cacheLeaderboardInfoItems'] = function(gameId, leaderboardList) {
		    console.log('LeaderboardOfflineCache: cacheLeaderboards');
			var dfd = $.Deferred();
			var args = {};
			var cacheRequests = [];
			
			$.each(leaderboardList, function(i, leaderboard) {
			    var leaderboardId = leaderboard['LeaderboardId'];
				var cacheRequest = {'primaryKey': "leaderMeta-" + leaderboardId,
			                        'secondaryKey': "leaderMeta-" + leaderboardId,
								    'jsonData': leaderboard};
			    cacheRequests.push(cacheRequest);
			});

			args['cacheRequests'] = cacheRequests;
			
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "setCacheItems", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: cacheLeaderboardInfoItems complete');
				/* TODO: Validate and decide on return type */
				dfd.resolve(responseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['clear'] = function() {
		    console.log('LeaderboardOfflineCache: getLeaderboardInfo');
			var dfd = $.Deferred();
			var args = {};
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "clear", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: getLeaderboardInfo complete: ' + JSON.stringify(responseJson));
				dfd.resolve(responseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['deleteLeaderboards'] = function(playerId, gameId) {
		    console.log('LeaderboardOfflineCache: deleteLeaderboards');
			var dfd = $.Deferred();
			var args = {};
			args['secondaryKey'] = "leaderMeta-" + playerId;
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "removeCachedItems", args: args});
			
			promise.done(function(responseJson, resultCode) {
			    console.log('LeaderboardOfflineCache: deleteLeaderboards complete: ' + JSON.stringify(responseJson));
				dfd.resolve(responseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['getHighestScoreSubmission'] = function(playerId, gameId, leaderboardId) {
		    console.log('LeaderboardOfflineCache: getHighestScoreSubmission');
			var dfd = $.Deferred();
			var args = {};
			
			args['primaryKey'] = "highestScoreSubmission-" + leaderboardId + "-" + playerId;
			var promise = NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "getCacheItem", args: args});
			
			promise.done(function(responseJson, resultCode) {
				var scoresResponseJson = {};
				if (resultCode == constants.NativeCallResultCode.SUCCESS && responseJson && responseJson['score']) {
				    scoresResponseJson = responseJson['score'];
				}
				dfd.resolve(scoresResponseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['getHighestScoreSubmissions'] = function(playerId) {
		    console.log('LeaderboardOfflineCache: getHighestScoreSubmission');
			var dfd = $.Deferred();
			var args = {};
			
			args['secondaryKey'] = "highestScoreSubmission-" + "-" + playerId;
			NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "queryCacheItems", args: args}).always(function(responseJson, resultCode) {
				var highestScoreSubmissionsReponseJson = {'HighestScoreSubmissions':[]};
				
				if (resultCode == constants.NativeCallResultCode.SUCCESS && responseJson && responseJson['queryResult']) {
					console.log('LeaderboardOfflineCache: getLeaderboards complete: ' + JSON.stringify(responseJson));
					highestScoreSubmissionsReponseJson = {'HighestScoreSubmissions':responseJson['queryResult']};
				}
				
				dfd.resolve(highestScoreSubmissionsReponseJson, resultCode);
			});
			
			return dfd.promise();
		};
		
		this['updateCachedScoreSubmission'] = function(playerId, gameId, leaderboardId, score) {
		    console.log('LeaderboardOfflineCache: updateCachedScoreSubmission');
			var args = {};
			var cacheRequest = {'primaryKey': "highestScoreSubmission-" + leaderboardId + "-" + playerId,
		   	     				'secondaryKey': "highestScoreSubmission-" + "-" + playerId,
								'jsonData': {'leaderboardId':leaderboardId,
								             'score':score}};
			args['cacheRequest'] = cacheRequest;
            // getHighestScoreSubmission
            // If it is null, just cache it
            // If it wasn't null, getLeaderboardInfo
            // If it is null, give up
            // If it wasn't null, determine if this score is better based on info.getMetadata().sortOrder
			// If it was better, cache it
			
		    var dfd = $.Deferred();
			var thisRef = this;
		    this.getHighestScoreSubmission(playerId, gameId, leaderboardId).always(function(getHighestScoreSubmissionResult, getHighestScoreSubmissionResultCode) {
				if (getHighestScoreSubmissionResultCode != constants.NativeCallResultCode.SUCCESS) {
				    NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "setCacheItem", args: args}).always(function(responseJson, resultCode) {
					    dfd.resolve(responseJson, resultCode);
				    });
				} else {
				    thisRef.getLeaderboardInfo(gameId, leaderboardId).always(function(getLeaderboardInfoResult, getLeaderboardInfoResultCode) {
					    if (resultCode != constants.NativeCallResultCode.SUCCESS || !getLeaderboardInfoResult || !getLeaderboardInfoResult['LeaderboardInfo']) {
						    dfd.resolve({}, constants.NativeCallResultCode.ERROR);
						} else {
						    var sortOrder = getLeaderboardInfoResult['LeaderboardInfo']['SortOrder'];
							if (sortOrder == 'DESCENDING') {
							    if (score > getHighestScoreSubmissionResult) {
									NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "setCacheItem", args: args}).always(function(responseJson, resultCode) {
					                    dfd.resolve(responseJson, resultCode);
				                    });
								} else {
								    dfd.resolve({}, constants.NativeCallResultCode.SUCCESS);
							    }
							} else {
							    if (score < getHighestScoreSubmissionResult) {
								    NativeTransport.serviceCall({target: constants.OfflineCacheTypes.LEADERBOARDS_CACHE, method: "setCacheItem", args: args}).always(function(responseJson, resultCode) {
					                    dfd.resolve(responseJson, resultCode);
				                    });
								} else {
								    dfd.resolve({}, constants.NativeCallResultCode.SUCCESS);
								}
							}
						}
					});
				}
			});
			
			return dfd.promise();
		};
    };
	
    return constructor;
}());
console.log("LeaderboardOfflineCache loaded.");
