/*
 * Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * AMAZON.COM CONFIDENTIAL
 */

var nativehost = {};
nativehost.postMessage = function(calldata, rid) {
    var serviceCall = {};
    serviceCall[constants.NativeCallKeys.NATIVE_CALL] = calldata.nativeCall;
    serviceCall[constants.NativeCallKeys.TARGET] = calldata.target;
    serviceCall[constants.NativeCallKeys.HTTP_METHOD] = calldata.httpMethod;
    serviceCall[constants.NativeCallKeys.METHOD] = calldata.method;
    serviceCall[constants.NativeCallKeys.PARAMETERS] = calldata.parameters;
    serviceCall[constants.NativeCallKeys.REQUEST_ID] = rid;
    serviceCall[constants.NativeCallKeys.AUTHENTICATE] = calldata[constants.NativeCallKeys.AUTHENTICATE];

    hostinterface.handleMessage(JSON.stringify(serviceCall));
};

NativeTransport = (function() {
    var _requestCounter = 0;
    var reqId2Dfd = {};
    var ftueHandler = null;
    var gcSettings = null;
    
    return {
        serviceCall : function (params) {
            var dfd = $.Deferred();
            gcSettings.getGuestMode().always(function(globalGuestMode) {
                /* No authenticated calls allowed while in guest mode. */
                /* TODO: This is a cheap hack, need to move caching to a separate call. */
                if (globalGuestMode && params.target != constants.OfflineCacheTypes.ACHIEVEMENTS_CACHE && params.target != constants.OfflineCacheTypes.LEADERBOARDS_CACHE && params.target != constants.OfflineCacheTypes.PLAYER_PROFILE_CACHE) {
                    if (params.guestModeMethod != null) {
                        console.log("Guest mode: calling " + params.guestModeMethod + " instead of " + params.method);
                        params.method = params.guestModeMethod;
                        NativeTransport.guestModeServiceCall(params).always(function(result, resultCode) {
                            dfd.resolve(result, resultCode);
                        });
                    } else {
                        console.log("Rejecting attempt to make authenticated call for " + params.method + " while in guest mode.");
                        dfd.reject({}, constants.NativeCallResultCode.AUTHORIZATION_ERROR);
                    }
                    return;
                }
                
                /* TODO: online check? */
                /* TODO: separate cache calls from network calls so they don't get rejected by these checks. */
                params.nativeCall = constants.NativeCallTypes.MAKE_SERVICE_CALL;
                params[constants.NativeCallKeys.AUTHENTICATE] = true;
                NativeTransport.callNative(params).always(function(result, resultCode) {
                    dfd.resolve(result, resultCode);
                });
            });
            return dfd.promise();
        },
        
        guestModeServiceCall : function (params) {
            params.nativeCall = constants.NativeCallTypes.MAKE_SERVICE_CALL;
            params[constants.NativeCallKeys.AUTHENTICATE] = false;
            return this.callNative(params);
        },
        
        /* TODO: Quick hack to avoid changing NativeTransport from static to constructor based. */
        setFTUEHandler : function (ftueHandlerRef) {
            ftueHandler = ftueHandlerRef;
        },

        /* TODO: Quick hack to avoid changing NativeTransport from static to constructor based. */
        setGCSettings : function (gcSettingsRef) {
            gcSettings = gcSettingsRef;
        },

        callNative : function (params) {
            var requestId = _requestCounter++;
            var target = params.target;
            var method = params.method;
            var parameters = params.args;
            var timeout = params.timeout;
            var defaultRet = params.defaultRet;
            
            var dfd = $.Deferred();
            reqId2Dfd[requestId] = dfd;

            if (timeout > 0) {
                setTimeout(function() {
                    delete reqId2Dfd[requestId];
                    dfd.reject({}, constants.NativeCallResultCode.NETWORK_ERROR);
                }, timeout);
            }
            
            var calldata = {
                target: target,
                method: method,
                httpMethod: params.httpMethod,
                nativeCall: params.nativeCall,
                parameters: parameters || {}
            };
            calldata[constants.NativeCallKeys.AUTHENTICATE] = params[constants.NativeCallKeys.AUTHENTICATE];
            
            var returnVal = nativehost.postMessage(calldata, requestId);
            return dfd.promise();
        },

        callNativeReply : function(params, requestId, resultCode) {
            if (requestId != undefined) {
                var dfd = reqId2Dfd[requestId];
                if (dfd) {
                    delete reqId2Dfd[requestId];
                    dfd.resolve(params, resultCode);
                }
            } else {
                console.log("NativeTransport: ERROR: callNativeReply called with undefined RID");
            }
        }
    };
}());

function receiveReply(reply) {
    if (reply != undefined) {
        NativeTransport.callNativeReply(reply.params, reply.__requestId, reply.resultCode);
    } else {
        console.log("NativeTransport: ERROR: Received undefined reply from native layer");
    }
}
console.log("NativeTransport loaded.");
