OfflineEventHandler = (function() {
    var constructor = function(achievementServiceProxyRef, leaderboardServiceProxyRef) {
	    var requestTypeHandled = 'OfflineEventRequest';
		var achievementService = achievementServiceProxyRef;
		var leaderboardService = leaderboardServiceProxyRef;
		
		this['getHandledType'] = function() {
		    return requestTypeHandled;
	    };
		
	    this['handleRequest'] = function(request) {
			var dfd = $.Deferred();
			
			if (!(request instanceof Request)) {
				console.log('OfflineEventHandler: handleRequest: input was not of type Request');
				var result = new Result(constants.NativeCallResultCode.REQUEST_ERROR, {});
		        dfd.resolve(result);
			} else {
			    console.log('OfflineEventHandler: handleRequest: message is: ' + JSON.stringify(request.getType()));
			    var requestType = request.getType();
				var requestParams = request.getParams();
				var actionCode = requestParams['action'];
				
				var reply = {};
				switch (actionCode) {
					case 'processOfflineEvent':
					    var offlineEvent = requestParams['event'];
						//var offlineEventParams = offlineEvent['params'];
						var offlineEventParams = offlineEvent;
						
						if (offlineEventParams['type'] == 'leaderboardEvent') {
							var score = offlineEventParams['PlayerScore'];
							var leaderboardId = offlineEventParams['leaderboardId'];
							var gameId = offlineEventParams['gameId'];
							
							leaderboardService.submitScore(gameId, leaderboardId, score).always(function(result, resultCode) {
								console.log('OfflineEventHandler: submitScore promise: ' + JSON.stringify(result) + ' resultCode: ' + resultCode);
								if (resultCode != constants.NativeCallResultCode.SUCCESS) {
									var replyCode = resultCode;
									var replyData = result;
									var result = new Result(replyCode, replyData);

									dfd.resolve(result);
								} else {
									var replyCode = constants.NativeCallResultCode.SUCCESS;
									var replyData = result;
									var result = new Result(replyCode, replyData);

									dfd.resolve(result);
								}
							});
						} else {
							var progress = Math.min(offlineEventParams['progress'], 100);
						
							achievementService.updateAchievementProgress(offlineEventParams['achievementId'], progress).always(function(result, resultCode) {
								console.log('OfflineEventHandler: updateAchievementProgress promise: ' + JSON.stringify(result) + ' resultCode: ' + resultCode);
								if (resultCode != constants.NativeCallResultCode.SUCCESS) {
									var replyCode = resultCode;
									var replyData = result;
									var result = new Result(replyCode, replyData);

									dfd.resolve(result);
								} else {
									var replyCode = constants.NativeCallResultCode.SUCCESS;
									var replyData = result;
									var result = new Result(replyCode, replyData);

									dfd.resolve(result);
								}
							});
						}
						break;
					default:
						console.log('OfflineEventHandler: handleRequest: actionCode not supported: ' + actionCode);
				        var result = new Result(constants.NativeCallResultCode.REQUEST_ERROR, {});
		                dfd.resolve(result);
				}
		    }
			
			console.log('OfflineEventHandler: handleRequest: end of call');
			return dfd.promise();
		};
    };

    return constructor;
}());
console.log("OfflineEventHandler loaded.");
