/*
 * Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * AMAZON.COM CONFIDENTIAL
 */

ProfileServiceProxy = (function() {
    var constructor = function() {

        /* public functions */
        this['getProfile'] = function() {
            return NativeTransport.serviceCall({target: "ProfilesService", method: "getDefaultProfile"});
        };

        this['updateProfile'] = function(profile) {
            /* Even though the parameter is named avatarLocation (URL to an image),
               it actually is an avatarId (name of image, generic string). */
            var args = {alias: profile.alias, avatarLocation: profile.avatarId};
            return NativeTransport.serviceCall({target: "ProfilesService", method: "update", httpMethod: "POST", args: args});
        };
        
        this['updateProfileHidden'] = function(hidden) {
            var args = {hidden: hidden};
            return NativeTransport.serviceCall({target: "ProfilesService", method: "hideprofile", httpMethod: "POST", args: args});
        };

        this['getAvatars'] = function() {
            var dfd = $.Deferred();
            var promise = NativeTransport.serviceCall({target: "ProfilesService", method: "getavatars"});
            promise.always(function(avatars, resultCode) {
                var results = [];
                if (avatars && avatars.avatars && avatars.avatarsMap) {
                    /* Maintain the order returned by profile service, but convert it into a list of objects. */
                    var idByUrl = {};
                    for (var avatarId in avatars.avatarsMap) {
                        idByUrl[avatars.avatarsMap[avatarId]] = avatarId;
                    }
                    
                    for (var i = 0; i < avatars.avatars.length; i++) {
                        var avatarUrl = avatars.avatars[i];
                        var avatarId = idByUrl[avatarUrl];
                        results.push({avatarId: avatarId, avatarUrl: avatarUrl});
                    }
                }
                
                dfd.resolve(results, resultCode);
            });
            return dfd.promise();
        };
    };

    return constructor;
}());
console.log("ProfileServiceProxy loaded.");
