RequestFactory = (function() {
    var constructor = function(apiHandler, achievementHandler, leaderboardHandler, profilesHandler, timePlayedHandler, offlineEventHandler, ftueHandler) {
        /* TODO: Add validation to ensure handlers are not undefined and are, in fact, handlers */

		this['createAPIRequest'] = function(requestData) {
		    var reqType = apiHandler.getHandledType();
			var reqParams = requestData;
		    var request = new Request(reqType, reqParams);
			return request;
		};
		
		/*************************************************************************************/
		/* Offline event processing requests                                                 */
		this['createProcessOfflineEventRequest'] = function(eventRequest) {
		    var reqType = offlineEventHandler.getHandledType();
			var reqParams = {};
			reqParams['action'] = 'processOfflineEvent'; /* TODO: Centralize these actions */
			reqParams['event'] = eventRequest;
		    var request = new Request(reqType, reqParams);
			return request;
		};
		/* End of offline event processing requests                                          */
        /*************************************************************************************/
		
		/*************************************************************************************/
		/* AchievementHandler requests                                                       */
		this['createGetAchievementRequest'] = function(achievementId, isCacheOnly) {
		    var reqType = achievementHandler.getHandledType();
			var reqParams = {};
			reqParams['action'] = 'getAchievement'; /* TODO: Centralize these actions */
			reqParams['cacheOnly'] = isCacheOnly;
			reqParams['achievementId'] = achievementId;
		    var request = new Request(reqType, reqParams);
			return request;
		};
		
		this['createGetAchievementsRequest'] = function(isCacheOnly) {
		    var reqType = achievementHandler.getHandledType();
			var reqParams = {};
			reqParams['action'] = 'getAchievements'; /* TODO: Centralize these actions */
			reqParams['cacheOnly'] = isCacheOnly;
		    var request = new Request(reqType, reqParams);
			return request;
		};
		
		this['createUpdateAchievementsRequest'] = function(achievementId, achievementProgress) {
		    var reqType = achievementHandler.getHandledType();
			var reqParams = {};
			reqParams['action'] = 'updateAchievement'; /* TODO: Centralize these actions */
			reqParams['achievementId'] = achievementId;
			reqParams['achievementProgress'] = achievementProgress;
		    var request = new Request(reqType, reqParams);
			return request;
		};
		/* End of AchievementHandler requests                                                */
        /*************************************************************************************/

		/*************************************************************************************/
		/* LeaderboardHandler requests                                                       */
		this['createGetLeaderboardsForGameRequest'] = function(gameId, isCacheOnly) {
		    var reqType = leaderboardHandler.getHandledType();
			var reqParams = {};
			reqParams['action'] = 'getLeaderboardsForGame'; /* TODO: Centralize these actions */
			reqParams['cacheOnly'] = isCacheOnly;
			reqParams['gameId'] = gameId;
		    var request = new Request(reqType, reqParams);
			return request;
		};
		
		this['createGetHighScoreRequest'] = function(gameId, leaderboardId, scope, isCacheOnly) {
		    var reqType = leaderboardHandler.getHandledType();
			var reqParams = {};
			reqParams['action'] = 'getHighScore'; /* TODO: Centralize these actions */
			reqParams['cacheOnly'] = isCacheOnly;
			reqParams['gameId'] = gameId;
			reqParams['leaderboardId'] = leaderboardId;
			reqParams['scope'] = scope;
		    var request = new Request(reqType, reqParams);
			return request;
		};
		
		this['createGetLeaderboardRanksRequest'] = function(gameId, leaderboardId, leaderboardFilter, startRank, count, friendsEnabled, isCacheOnly) {
		    var reqType = leaderboardHandler.getHandledType();
			var reqParams = {};
			reqParams['action'] = 'getLeaderboardRanks'; /* TODO: Centralize these actions */
			reqParams['cacheOnly'] = isCacheOnly;
			reqParams['gameId'] = gameId;
			reqParams['leaderboardId'] = leaderboardId;
			reqParams['scope'] = leaderboardFilter;
			reqParams['startRank'] = startRank;
			reqParams['numRanks'] = count;
			reqParams['FriendsEnabled'] = friendsEnabled;
		    var request = new Request(reqType, reqParams);
			return request;
		};
		
		this['createGetLeaderboardPercentilesRequest'] = function(gameId, leaderboardId, leaderboardFilter, friendsEnabled, isCacheOnly) {
		    var reqType = leaderboardHandler.getHandledType();
			var reqParams = {};
			reqParams['action'] = 'getLeaderboardPercentiles'; /* TODO: Centralize these actions */
			reqParams['cacheOnly'] = isCacheOnly;
			reqParams['gameId'] = gameId;
			reqParams['leaderboardId'] = leaderboardId;
			reqParams['scope'] = leaderboardFilter;
			reqParams['FriendsEnabled'] = friendsEnabled;
		    var request = new Request(reqType, reqParams);
			return request;
		};
		
		this['createSubmitScoreRequest'] = function(leaderboardId, playerScore) {
		    var reqType = leaderboardHandler.getHandledType();
			var reqParams = {};
			reqParams['action'] = 'submitScore'; /* TODO: Centralize these actions */
			reqParams['leaderboardId'] = leaderboardId;
			reqParams['PlayerScore'] = playerScore;
		    var request = new Request(reqType, reqParams);
			return request;
		};
		
		/* TODO: Remove this when we change the guest experience on the LB pages */
		this['createGetHighestScoreSubmissionsRequest'] = function() {
		    var reqType = leaderboardHandler.getHandledType();
			var reqParams = {};
			reqParams['action'] = 'getHighestScoreSubmissions'; /* TODO: Centralize these actions */
		    var request = new Request(reqType, reqParams);
			return request;
		};

		/* End of LeaderboardHandler requests                                                */
        /*************************************************************************************/

        /*************************************************************************************/
        /* ProfileHandler requests                                                           */
        this['createGetProfileRequest'] = function(isCacheOnly) {
            var reqType = profilesHandler.getHandledType();
            var reqParams = {};
            reqParams['action'] = 'getProfile'; /* TODO: Centralize these actions */
            reqParams['cacheOnly'] = isCacheOnly;
            var request = new Request(reqType, reqParams);
            return request;
        };

        this['createUpdateProfileRequest'] = function(profile) {
            var reqType = profilesHandler.getHandledType();
            var reqParams = {};
            reqParams['action'] = 'updateProfile'; /* TODO: Centralize these actions */
            reqParams['profile'] = profile;
            var request = new Request(reqType, reqParams);
            return request;
        };
        
        this['createUpdateProfileHiddenRequest'] = function(hidden) {
            var reqType = profilesHandler.getHandledType();
            var reqParams = {};
            reqParams['action'] = 'updateProfileHidden'; /* TODO: Centralize these actions */
            reqParams['hidden'] = hidden;
            var request = new Request(reqType, reqParams);
            return request;
        };
        
        this['createGetAvatarsRequest'] = function(isCacheOnly) {
            var reqType = profilesHandler.getHandledType();
            var reqParams = {};
            reqParams['action'] = 'getAvatars'; /* TODO: Centralize these actions */
            reqParams['cacheOnly'] = isCacheOnly;
            var request = new Request(reqType, reqParams);
            return request;
        };
        /* End of ProfileHandler requests                                                    */
        /*************************************************************************************/

        /*************************************************************************************/
        /* TimePlayed requests                                                               */
        this['createGetTimePlayedRequest'] = function(isCacheOnly) {
            var reqType = timePlayedHandler.getHandledType();
            var reqParams = {};
            reqParams['action'] = 'getTimePlayed'; /* TODO: Centralize these actions */
            reqParams['cacheOnly'] = isCacheOnly;
            var request = new Request(reqType, reqParams);
            return request;
        };
        /* End of TimePlayed requests                                                        */
        /*************************************************************************************/

        /*************************************************************************************/
        /* FTUE requests                                                                     */
        
        /* Star the login workflow.  This doesn't launch the browser immediately. */
        this['createLoginRequest'] = function() {
            var reqType = ftueHandler.getHandledType();
            var reqParams = {};
            reqParams['action'] = 'login'; /* TODO: Centralize these actions */
            var request = new Request(reqType, reqParams);
            return request;
        };

        this['createLogoutRequest'] = function() {
            var reqType = ftueHandler.getHandledType();
            var reqParams = {};
            reqParams['action'] = 'logout'; /* TODO: Centralize these actions */
            var request = new Request(reqType, reqParams);
            return request;
        };
        /* End of TimePlayed requests                                                        */
        /*************************************************************************************/
	};
			
    return constructor;
}());
console.log("RequestFactory loaded.");
