ServiceFactory = (function() {
    /* Private variables */
	
	/* ACHIEVEMENTS */
	var achievementOfflineCache = new AchievementOfflineCache();
	var achievementServiceProxy = new AchievementServiceProxy();
	
    var achievementHandlerGuest = new AchievementHandlerGuest(achievementServiceProxy, achievementOfflineCache);
	var achievementHandlerRecognized = new AchievementHandlerRecognized(achievementServiceProxy, achievementOfflineCache);
	var achievementHandler = new AchievementHandler(achievementHandlerGuest, achievementHandlerRecognized);
	
	/* LEADERBOARDS */
	var leaderboardServiceProxy = new LeaderboardServiceProxy();
	var leaderboardOfflineCache = new LeaderboardOfflineCache();
	
	var leaderboardHandlerGuest = new LeaderboardHandlerGuest(leaderboardServiceProxy, leaderboardOfflineCache);
	var leaderboardHandlerRecognized = new LeaderboardHandlerRecognized(leaderboardServiceProxy, leaderboardOfflineCache);
	var leaderboardHandler = new LeaderboardHandler(leaderboardHandlerGuest, leaderboardHandlerRecognized);
	
	/* PROFILES */
    var profilesHandler = new ProfilesHandler(new ProfileServiceProxy(), new ProfileOfflineCache());

	/* FTUE + SETTINGS */
    var countries = new Countries();
    var gcSettings = new GCSettings();
    var ftueHandler = new FTUEHandler(profilesHandler, countries, gcSettings);
    NativeTransport.setFTUEHandler(ftueHandler);
    NativeTransport.setGCSettings(gcSettings);

	/* TIME PLAYED */
    var timePlayedHandler = new TimePlayedHandler(ftueHandler.getInitializationPromise());
	
	/* SDK API */
    var apiHandler = new APICallHandler(ftueHandler, timePlayedHandler, gcSettings);
	
	/* OFFLINE EVENT PROCESSING */
	var offlineEventHandler = new OfflineEventHandler(achievementServiceProxy, leaderboardServiceProxy);

	/* Register handlers */
    var requestRouter = new RequestRouter();
    requestRouter.registerHandler(apiHandler.getHandledType(), apiHandler);
    requestRouter.registerHandler(achievementHandler.getHandledType(), achievementHandler);
    requestRouter.registerHandler(leaderboardHandler.getHandledType(), leaderboardHandler);
    requestRouter.registerHandler(profilesHandler.getHandledType(), profilesHandler);
    requestRouter.registerHandler(timePlayedHandler.getHandledType(), timePlayedHandler);
    requestRouter.registerHandler(ftueHandler.getHandledType(), ftueHandler);
	requestRouter.registerHandler(offlineEventHandler.getHandledType(), offlineEventHandler);

    requestFactory = new RequestFactory(apiHandler, achievementHandler, leaderboardHandler, profilesHandler, timePlayedHandler, offlineEventHandler, ftueHandler);

    return {
        getRequestFactory : function () {
		    return requestFactory;
        },

        getRequestRouter : function() {
		    return requestRouter;
        },
        
        getTimePlayedHandler : function() {
            return timePlayedHandler;
        },
        
        getGCSettings : function() {
            return gcSettings;
        }
    };
}());

function printStackTrace() { 
    function stackTraceHelper(targetFunction) {
        if (!targetFunction) {
            return [];
        }
        
        /* Convert array-like object into array for join to work. */
        var tmp = [];
        for (var i=0; i<arguments.length; i++) {
            tmp.push(arguments[i]);
        }
        var args = tmp.join();

        return stackTraceHelper(targetFunction.caller).concat([args]);
    }

    var result = stackTraceHelper(arguments.callee.caller);
    for (var i=0; i<result.length; i++) {
        console.log("------------------------------------------\n" + result[i]);
    }
};
console.log("ServiceFactory loaded.");
